;;; A finite state machine for going through the experiment

;; intro
;;  show instructions
;; 

;; A page is something the user sees. There are different kinds of
;; pages. a problem page displays a problem. The user must complete
;; the problem to pass.
;;
;; A text page displays instructions.
;;
;; end page displays 'end'
;;  

;; (list
;;  '(:instruction "abc" "def")
;;  `(:problems
;;    ,(make-problem '(* (+ 5 7) 9) 108)))
  

;;(defvar *fsm-state-stack* nil)

(in-package :tutor)

(defvar *fsm-current-state* nil)

(defun get-fsm-block-fn (name)
  (intern (format nil "FSM-BLOCK-~a" (symbol-name name))))

(defgeneric fsm-init (fsm state))

(defgeneric fsm-end (fsm))

(defmethod fsm-end (blk)
  (declare (ignore blk))
  (delete-subviews *window*))

(defgeneric fsm-dump-screen (fsm))

(defgeneric fsm-execute-command (fsm cmd args))

;; (defmacro define-fsm-block (name block-init block-end)
;;   (let ((sym (get-fsm-block-fn name))
;; 	(cmd (gensym))
;; 	(state (gensym)))
;;   `(defun ,sym (,cmd ,state)
;;      (cond ((eq ,cmd :init)
;; 	    (funcall ,block-init ,state))
;; 	   ((eq ,cmd :end)
;; 	    (funcall ,block-end))
;; 	   (t (error "unknown cmd ~a" ,cmd))))))

;; (defun fsm-add-states (states)
;;   "Append states to the end of the fsm state list."
;;   (

;; (defun fsm-compile (states)
;;   (cond ((null states) nil)
;; 	((eq (caar states) :instruction)
;; 	 (lambda ()
;; 	   (fsm-block-instruction :init (cdar states))))
;; 	((eq (caar states) :problems)
;; 	 (lambda ()
;; 	   (

(defun fsm-block-change (state cmd)
  (let ((fn (get-fsm-block-fn (car state))))
    (if (fboundp fn)
	(funcall fn cmd (cdr state)))))

(defun fsm-next ()
  (fsm-end (first (car *fsm-current-state*)))
  (pop *fsm-current-state*)
  (fsm-init (first (car *fsm-current-state*)) (rest (car *fsm-current-state*))))

(defun fsm-start (states)
  (setf *fsm-current-state* states)
  (fsm-init (first (car *fsm-current-state*)) (rest (car *fsm-current-state*))))

(defmethod fsm-init ((blk (eql 'end)) states)
  (declare (ignore blk states))
  (history-end)
  (let ((str "End of Experiment."))
    (make-instance 'static-text-dialog-item
		   :view-container *window*
		   :view-position (make-point (-  (truncate (view-width *window*) 2) 
						  (truncate (string-width str) 2))
					      (truncate (view-height *window*) 2))
		   :dialog-item-text str)
    (format t "~&==============~%Score is: ~a~%Payment due so far: ~,2f~%==============~%" (tutor-score *tutor*) (calculate-money (tutor-score *tutor*)))))

(defmethod fsm-dump-screen ((blk (eql 'end)))
  (list (list :text "End of Experiment.")))

(defmethod fsm-execute-command ((blk (eql 'end)) (cmd (eql :click)) target)
  (error "Unknown target ~a" target))

;; (defmethod fsm-end ((blk (eql 'end)) state)
;;   (delete-subviews *window*))

;; (define-fsm-block end 'fsm-end-init 'fsm-end-end)

(defun tutor (id states)
  (init)
  ;; Demerits are cummulative across problems
  ;;(setf (tutor-total-points *tutor*) score)
  (setf (tutor-score *tutor*) 0)
  (history-init id)
  (history-add-start-tutor)
  (fsm-start states))

;;;

(defun execute-command (cmd args)
  (fsm-execute-command (first (car *fsm-current-state*)) cmd args))

(defun dump-screen ()
  (fsm-dump-screen (first (car *fsm-current-state*))))


;;; section break

(defmethod fsm-init ((blk (eql 'break)) states)
  (declare (ignore blk states))
  (let* ((view (make-instance 'view
                              :view-container *window*))
         (txt (make-instance 'static-text-dialog-item
                             :view-container view
                             :view-position #@(0 0)
                             :dialog-item-text (format nil "~d sections to go!~%Click Continue to go to the next section." (fsm-sections-left *fsm-current-state*))))
         (but (make-instance 'button-dialog-item
                             :view-container view
                             :dialog-item-text "Continue"
                             :dialog-item-action (button-fn (fsm-next)))))
    (set-view-size view (make-point (max (point-h (view-size txt))
                                         (point-h (view-size but)))
                                    (+ (point-v (view-size txt))
                                       (point-v (view-size but))
                                       15)))
    (set-view-position view (make-point (- (truncate (view-width *window*) 2)
                                           (truncate (point-h (view-size view)) 2))
                                        (- (truncate (view-height *window*) 2)
                                           (truncate (point-v (view-size view)) 2))))
    (set-view-position but (make-point (- (truncate (point-h (view-size view)) 2)
                                          (truncate (point-h (view-size but)) 2))
                                       (+ (point-v (view-size txt))
                                          10)))
    (history-add-break)))

(defun fsm-sections-left (states)
  (1- (length (remove-if (lambda (x) (eq (first x) 'problem)) states))))

(defmethod fsm-dump-screen ((blk (eql 'break)))
  (list (list :text (format nil "~d sections to go!~%Click Continue to go to the next section." (fsm-sections-left *fsm-current-state*)))
        (list :buttons :continue)))

(defmethod fsm-execute-command ((blk (eql 'break)) cmd args)
  (if (and (eq cmd :click)
          (eq args :continue))
      (fsm-next)
    (error "~a or ~a is not a valid option." cmd args)))
