;;; Messages, Hints, Score

(in-package :tutor)

(defvar *message-widget* nil)
(defvar *hint-text* nil)
(defvar *message-text* nil)

(defun clear-message (&optional txt-too)
  (when *message-widget*
    (remove-subviews (view-container *message-widget*) *message-widget*)
    (setf *message-widget* nil))
  (when txt-too
    (setf *hint-text* nil
          *message-text* nil)))

(defun redraw-message ()
  (clear-message)
  (let* ((msg (or *hint-text* *message-text*))
         (err (if *hint-text*
                  *message-text*
                nil)))
    (when msg
      (setf *message-widget* (if err
                                 (draw-text-plus-header (if (and (tutor1-p) 
                                                                 (not *simplified-ui*))
                                                            *window*
                                                          *message-view*)
                                                        0 0 err msg '("Monaco" :bold 12))
                               (draw-text-with-font (if (and (tutor1-p)
                                                             (not *simplified-ui*))
                                                        *window*
                                                      *message-view*)
                                                    0 0 msg '("Monaco" :bold 12))))
      (if (tutor1-p)
        (if *simplified-ui*
            (set-view-position *message-widget* 0 0)
          (set-view-position *message-widget*
                             555
                             (- (point-v (add-points (view-position *message-view*)
                                                     (view-size *message-view*)))
                                (point-v (view-size *message-widget*)))))
        (if *simplified-ui*
            (set-view-position *message-widget* 0 0)
          (set-view-position *message-widget*
                             5 5))))))

(defun init-message-widget ()
  (setf *message-widget* nil))

(defun set-hint-text (txt)
  "Set hint portion of text-above-the-problem"
  (setf *hint-text* txt)
  (redraw-message))

(defun set-message-text (txt)
  "Set message portion of text-above-the-problem"
  (setf *message-text* txt)
  (redraw-message))

(defun calculate-money (score)
  (format nil "$~,2F" ;;(* (/ score *total-points*) (* 32 (/ *total-points* +total-points+)))))
          (/ score 100)))

(defun draw-score ()
  (when (tutor-score-view *tutor*)
    (let ((v (draw-text (tutor-score-view *tutor*) 0 0 (calculate-money (tutor-score *tutor*)))))
      (set-view-position v (- (view-width (tutor-score-view *tutor*)) (view-width v)) 0))))

(defun draw-problems ()
  (when (tutor-problems-view *tutor*)
    (let ((v (draw-text (tutor-problems-view *tutor*) 0 0 (format nil "Problem ~a of ~a" 
                                                                  (- (tutor-num-problems *tutor*) (length (problems (tutor-problem-set *tutor*))))
                                                                  (tutor-num-problems *tutor*)))))
      (set-view-position v (- (view-width (tutor-problems-view *tutor*)) (view-width v)) 0))))

(defun draw-stats ()
  (draw-score)
  (draw-problems))