;;; Helper functions and operations 

(in-package :tutor)


;; top => arg
;; box => (:arg :op op :arg1 arg :arg2 arg :permute bool) | (:answer :value arg) | (:x) | (:number :value arg) | (:equals :arg1 arg :arg2 arg)
;; op => (:matches op-matches :name name)
;; arg => (:matches matches :name name)
;; name => symbol
;; op-matches => (op*)
;; op => * / = + -
;; matches => :box | :number | :any | box 

;; '(:matches (:arg :op (:matches (=) :name op)
;;                  :arg1 (:matches :box :name arg1)
;;                  :arg2 (:matches :box :name arg2))
;;   :name box)

;; (defvar *green-box-hints* nil
;;   "Hints supplied by the operation.")

(defun fmt (fmt &rest args)
  (apply #'format nil fmt args))

(defun set-hint (t1-hint t2-hint)
  "Set the operation hint."
  (setf (tutor-green-box-hints *tutor*) (if (tutor1-p)
                                            t1-hint
                                          t2-hint)))

(defun set-both-hints (hint)
  (setf (tutor-green-box-hints *tutor*) hint))

(defun make-answer-set ()
  '())

(defmacro add-answerf (set box answers)
  "add (box answer) to set."
  (let ((cache (gensym)))
    `(let ((,cache ,answers))
       (if (assq ,box ,set)
           (setf (cdr (assq ,box ,set)) (nconc (cdr (assq ,box ,set)) ,cache))
         (push (cons ,box ,cache) ,set)))))



(defun matcher (pat tree)
  (let ((symbols '()))
    (labels ((addsym (sym val)
               (let ((a (getf symbols sym)))
                 ;; If its already in the symbol table its gotta have
                 ;; the same value.
                 (if a
                     (eql a val)
                   (setf (getf symbols sym) val))))

             (match-op (pat op)
               (when (find op (getf pat :matches))
                 (addsym (getf pat :name) op)))

             (match-arg (pat arg)
               (multiple-value-bind (succ syms) (matcher pat arg)
                 (when succ
                   ;; This is a bit gross. We Add each of the pairs to
                   ;; the list, verifying that syms of the same name
                   ;; have the same value. If not, return nil.
                   (do* ((s syms (cddr s))
                         (v (cdr s) (cdr s)))
                       ((null s) t)
                     (addsym (car s) (car v))))))

             (match-box (props op arg1 arg2)
               (and (match-op (getf props :op) op)
                    (match-arg (getf props :arg1) arg1)
                    (match-arg (getf props :arg2) arg2)))

             ;; Handle the BOX rule
             (match (pat box)
               (let ((props (rest pat)))
                 (ecase (first pat)
                   (:arg 
                    (when (typep box 'arg-box)
                      (let ((bk (copy-list symbols))
                            (try (match-box props
                                            (arg-value (box-op box))
                                            (arg-value (box-arg1 box))
                                            (arg-value (box-arg2 box)))))
                        (if (and (getf props :permute)
                                 (not try))
                            (progn
                              ;; Restore the symbols and try with the args reversed
                              (setf symbols bk)
                              (match-box props
                                         (arg-value (box-op box))
                                         (arg-value (box-arg2 box))
                                         (arg-value (box-arg1 box))))
                              try))))
                   (:equals
                    (when (typep box 'equal-box)
                      (let ((bk (copy-list symbols))
                            (try (and (match-arg (getf props :arg1) (arg-value (box-arg1 box)))
                                      (match-arg (getf props :arg2) (arg-value (box-arg2 box))))))
                        (if (and (getf props :permute)
                                 (not try))
                            (progn
                              ;; Restore the symbols and try with the args reversed
                              (setf symbols bk)
                              (and (match-arg (getf props :arg1) (arg-value (box-arg2 box)))
                                   (match-arg (getf props :arg2) (arg-value (box-arg1 box)))))
                            try))))
                   (:x
                    (typep box 'x-box))
                   (:answer 
                    (when (typep box 'answer-box)
                      (match-arg (getf props :value) (arg-value (answer box)))))
                   (:number
                    (when (typep box 'number-box)
                      (match-arg (getf props :value) (arg-value (number box)))))
                   (:or
                    (loop for i in props
                          thereis (matcher `(:matches ,i) box)))))))
      (cond
       ((eq (getf pat :matches) :box)
        ;; a box means anything but an answer box. answer boxes are a
        ;; special hack.
        (when (and (typep tree 'box)
                   (not (typep tree 'number-box)))
          (values t 
                  (list (getf pat :name) tree))))
       ((eq (getf pat :matches) :number)
        (when (typep tree 'number)
          (values t 
                  (list (getf pat :name) tree))))
       ((eq (getf pat :matches) :any)
        (values t 
                (list (getf pat :name) tree)))
       ((numberp (getf pat :matches))
        (when (eql (getf pat :matches) tree)
          (values t 
                  (list (getf pat :name) tree))))
       ((listp (getf pat :matches))
        (when (match (getf pat :matches) tree)
          (addsym (getf pat :name) tree)
          (values t
                  symbols)))
       (t (error "huh"))))))

(defun looking-at (tree pattern)
  "Return T if TREE matches the match expression PATTERN."
  (if (matcher pattern tree) t nil))

;; root = :n | :b | :x | list
;; list = (op root root)
;; op = +-*/ :a :m
;; :n - number
;; :b - any box
;; :x - the x box
(defun simple-match (tree pattern)
  (cond ((listp pattern)
         (when (typep tree 'arg-box)
           (and (case (first pattern)
                  (:a
                   (find (arg-value (box-op tree)) '(+ -)))
                  (:m
                   (find (arg-value (box-op tree)) '(* /)))
                  (t
                   (eq (first pattern) (arg-value (box-op tree)))))
                (simple-match (arg-value (box-arg1 tree)) (second pattern))
                (simple-match (arg-value (box-arg2 tree)) (third pattern)))))
        ((eq pattern :n)
         (numberp tree))
        ((eq pattern :b)
         (typep tree 'box))
        ((eq pattern :x)
         (typep tree 'x-box))
        (t (error "I don't know how to match ~a" pattern))))
           
(defun zip (a b)
  "zip lists a and b together: ((a b) (c d) ..)"
  (mapcar #'list a b))

(defun unzip (plist)
  "Return 2 values, the list of symbols and the list of values in plist."
  (do* ((syms nil (cons s syms))
        (vals nil (cons v vals))
        (l plist (cddr l))
        (s (first l) (first l))
        (v (second l) (second l)))
      ((null l) (values syms vals))))

(defun plist-vals (plist syms)
  (cond ((null syms) nil)
        (t (multiple-value-bind (s v tail) (get-properties plist syms)
             (cons v (plist-vals tail (remove s syms)))))))

(defun boxes-in-plist (plist)
  (multiple-value-bind (s v) (unzip plist)
    (declare (ignore s))
    (only-boxes v)))

(defun all-boxes-in-plist-p (plist boxes)
  "Verify that all boxes in alist are in boxes and vice versa."
  (not (set-exclusive-or (boxes-in-plist plist)
                         boxes)))

(defmacro with-matches ((&rest vars) plist &body body)
  `(let ,(mapcar (lambda (x) `(,x (getf ,plist ',x))) vars)
     ,@body))

;; (new-subtree (* 

(defun new-subtree (tree root)
  "Inject a subtree into the diagram. tree is parsed and set to
  hang from ARG. tree is considered to be editable content so all
  numbers and operators will be editable. Returns the answer set."
  (let ((answers (make-answer-set)))
    (labels ((parse (tree parent)
                    (cond ((numberp tree)
                           ;;(dformat "moooo~%")
                           tree)
;;                           ((symbolp tree)
;;                            ;; Add the parent
;;                            (push parent (box-parents x-box))
;;                            x-box)
                          ((typep tree 'box)
                           ;;(dformat "haaa~%")
                           ;; fix up the parent. The caller has to remove the old parent
                           (push parent (box-parents tree))
                           tree)
                          ((find (first tree) '(+ / * -))
                           (let* ((box (make-instance 'arg-box
                                                      :parents (list parent)
                                                      :op (make-instance 'operator-arg)
                                                      :arg1 (make-instance 'argument)
                                                      :arg2 (make-instance 'argument)))
                                  (b1 (parse (second tree) box))
                                  (b2 (parse (third tree) box)))
                             ;;(dformat "heee~%")
                             ;; set up values and editability
                             (setf (arg-value (box-arg1 box)) b1
                                   (arg-editable (box-arg1 box)) (numberp b1)
                                   (arg-value (box-arg2 box)) b2
                                   (arg-editable (box-arg2 box)) (numberp b2)
                                   ;; ops are always editable
                                   (arg-answer (box-op box)) (first tree)
                                   (arg-editable (box-op box)) t)
                             ;; FIXME: reparent?
                             ;; Setup answers
                             (when (arg-editable (box-arg1 box))
                               (setf (arg-answer (box-arg1 box)) b1))
                             (when (arg-editable (box-arg2 box))
                               (setf (arg-answer (box-arg2 box)) b2))
                             ;; Add to answerset
                             (add-answerf answers box (op-answers (first tree) b1 b2))
                             box))
                          (t (error "hoeusntho")))))
      (let ((new-tree (parse tree root)))
        (values new-tree answers)))))

(defun do-submit (&rest answer-sets)
  "Call this at the end of the operation."
  ;; (dformat "~a ~a~%" answer-sets (mapcar (lambda (answers) (gen-answer (tutor-eqn-root *tutor*) answers)) answer-sets))
  (setf (tutor-submit-enabled *tutor*) t
        (tutor-submit-answers *tutor*) (mapcar (lambda (answers) (gen-answer (tutor-eqn-root *tutor*) answers)) answer-sets)))

(defun symbols (tree)
  (cond ((null tree) nil)
	((numberp tree) nil)
	((symbolp tree)
	 (unless (member tree '(+ - / * =))
	   (list tree)))
	(t (nconc (symbols (first tree))
		  (symbols (second tree))
		  (symbols (third tree))
		  (symbols (fourth tree))))))

(defstruct operation
  t1-name t2-name t1-quick-key t2-quick-key patterns selects fn)

(defparameter *operations* nil)

(defun all-operations (&optional (keywordify t))
  (loop for i in *operations*
        collect (let ((n (if (tutor1-p)
                             (operation-t1-name i)
                           (operation-t2-name i))))
                  (if keywordify
                      (intern (symbol-name n) "KEYWORD")
                    n))))

(defun op-name (op)
  (string-capitalize (substitute #\Space #\- (symbol-name (operation-name op)))))

(defgeneric find-operation (criteria))

(defmethod find-operation ((ch character))
  (if (tutor1-p)
      (find ch *operations* :test #'char= :key #'operation-t1-quick-key)
    (find ch *operations* :test #'char= :key #'operation-t2-quick-key)))

(defmethod find-operation ((sym symbol))
  (if (tutor1-p)
      (find sym *operations* :key #'operation-t1-name)
    (find sym *operations* :key #'operation-t2-name)))

(defun operation-quick-key (op)
  (if (tutor1-p)
      (operation-t1-quick-key op)
    (operation-t2-quick-key op)))

(defun operation-name (op)
  (if (tutor1-p)
      (operation-t1-name op)
    (operation-t2-name op)))

(defmacro defoperation (t1-name t1-quick-key t2-name t2-quick-key (plist nth-match (&rest selected) (&rest patterns)) &body body)
  "Create an operation."
  (let ((boxes (gensym "BOXES"))
	(matched (gensym "MATCHES"))
        (pat (gensym "PAT"))
	(root (gensym "ROOT")))
    `(progn
       (setf *operations*
             (append *operations*
                     (list (make-operation
                            :t1-name ',t1-name 
                            :t1-quick-key ,t1-quick-key
                            :t2-name ',t2-name 
                            :t2-quick-key ,t2-quick-key
                            :patterns ',patterns
                            :selects ',selected
                            :fn (lambda ()
                                  (let* ((,boxes (selected-boxes))
                                         (,root (find-root-verify-rest ,boxes)))
                                    (when ,boxes
                                      (loop for ,pat in ',patterns
                                            for ,nth-match from 0
                                            do (multiple-value-bind (,matched ,plist) (matcher ,pat ,root)
                                                 (when (and ,matched
                                                            (null (set-exclusive-or (plist-vals ,plist (nth ,nth-match ',selected)) ,boxes)))
                                                   ,@body
                                                   (return ,t))))))))))))))

;; (= (+ (- 8) (* (- 2) X)) 0) ==> (= (-2 * X) (- 0 -8))

(defmacro def-op-answers (op op1 op2 &key nn on no oo)
  "Create a function that returns the possible combinations of operands for OP."
  (let ((reverse-p (gensym))
        (fn (intern (format nil "~a-ANSWER" (symbol-name op)))))
    `(defun ,fn (,op1 ,op2 &optional (,reverse-p ,(or (eql op '+)
                                                      (eql op '*))))
       "Return a list of possible answers involving op1 ond op2."
       (nconc (cond ((and (numberp ,op1) 
                          (numberp ,op2))
                     ,nn)
                    ((and (not (numberp ,op1))
                          (numberp ,op2))
                     ,on)
                    ((and (numberp ,op1)
                          (not (numberp ,op2)))
                     ,no)
                    ((and (not (numberp op1))
                          (not (numberp op2)))
                     ,oo))
              (when ,reverse-p
		(,fn ,op2 ,op1 nil))))))

(def-op-answers + op1 op2
  :nn (list (list '+ op1 op2)
            (list '- op1 (- op2)))
  :on (list (list '+ op1 op2)
            (list '- op1 (- op2)))
  :no (list (list '+ op1 op2))
  :oo (list (list '+ op1 op2)))

(def-op-answers - op1 op2
  :nn (list (list '- op1 op2)
            (list '+ op1 (- op2))
            (list '+ (- op2) op1))
  :on (list (list '- op1 op2)
            (list '+ op1 (- op2))
            (list '+ (- op2) op1))
  :no (list (list '- op1 op2))
  :oo (list (list '- op1 op2)))

(def-op-answers / op1 op2
  :nn (list (list '/ op1 op2)
            (list '* op1 (/ 1 op2))
            (list '* (/ 1 op2) op1)
            (list '* (- op1) (- (/ 1 op2)))
            (list '* (- (/ 1 op2)) (- op1))
            (list '/ (- op1) (- op2)))
  :on (list (list '/ op1 op2)
            (list '* op1 (/ 1 op2)))
  :no (list (list '/ op1 op2))
  :oo (list (list '/ op1 op2)))

(def-op-answers * op1 op2
  :nn (list (list '* op1 op2)
            (list '* (- op1) (- op2))
            (list '/ op1 (/ 1 op2))
            (list '/ (- op1) (- (/ 1 op2))))
  :on (list (list '* op1 op2)
            (list '/ op1 (/ 1 op2)))
  :no (list (list '* op1 op2))
            ;;(list '/ (/ 1 op1) op2))
  :oo (list (list '* op1 op2)))

(def-op-answers = op1 op2
  :nn (list (list '= op1 op2)
            (list '= op2 op1))
  :on (list (list '= op1 op2)
            (list '= op2 op1))
  :no (list (list '= op1 op2)
            (list '= op2 op1))
  :oo (list (list '= op1 op2)
            (list '= op2 op1)))


(defun op-answers (op op1 op2)
  (case op
    (+ (+-answer op1 op2))
    (- (--answer op1 op2))
    (/ (/-answer op1 op2))
    (* (*-answer op1 op2))
    (= (=-answer op1 op2))))

(defun set-arg-box-answer (box op a1 a2)
  (setf (arg-answer (box-op box)) op
        (arg-answer (box-arg1 box)) a1
        (arg-answer (box-arg2 box)) a2))

(defmacro iflet (sym expr &body body)
  "if expr is true, bind it to sym and eval body. otherwise, nil."
  `(let ((,sym ,expr))
     (when ,sym
       ,@body)))

(defun get-arg-box-answer (box)
  (let (accum)
    (labels ((push-arg (arg)
                       (when (arg-editable arg)
                         (push (arg-answer arg) accum))))
      (push-arg (box-arg2 box))
      (push-arg (box-op box))
      (push-arg (box-arg1 box))
      (iflet s accum
        (if (= (length s) 3)
            (list (format nil "~{~a~^ ~}" s))
          s)))))

(defun editables-answers (root)
  (cond ((typep root 'arg-box)
         (nconc
          (iflet a (get-arg-box-answer root)
            a)
          (apply #'nconc (mapcar #'editables-answers (children root)))))
        ((typep root 'answer-box)
         (nconc (when (arg-editable (answer root))
                  (list (arg-answer (answer root))))
                (apply #'nconc (mapcar #'editables-answers (children root)))))
        (t nil)))


;; 3 + 3X - (6X + 12) - (X + 1) = 0
;; <==> 3 + 3X - (2 * (3X + 6) - (X + 1)) = 0

(defun gen-answer (root box-alist)
  (let ((b (assq root box-alist)))
    ;; Override root if root is in the alist
    (if b
        (apply #'nconc (mapcar (lambda (x) (gen-answer x box-alist)) (cdr b)))
      ;; Turn root into a list, store it in eqn
      (let ((eqn (cond ((or (typep root 'number)
                            (typep root 'symbol)
                            (typep root 'list))
                        root)
                       ((typep root 'x-box)
                        'x)
                       ((typep root 'answer-box)
                        (list (arg-value (answer root))))
                       ((typep root 'arg-box)
                        (list (arg-value (box-op root))
                              (arg-value (box-arg1 root))
                              (arg-value (box-arg2 root))))
                       ((typep root 'number-box)
                        (list (arg-value (number root))))
                       (t (error "huh? ~a" (type-of root))))))
        ;; now process eqn
        (cond ((or (numberp eqn)
                   (symbolp eqn))
               (list eqn))
              ((= (length eqn) 1)
               ;; For each tree returned wrap it in a list to denote
               ;; that its an answer box. Return the list of trees.
               (mapcar #'list (gen-answer (first eqn) box-alist)))
              ((= (length eqn) 3)
               (let* ((lhs (gen-answer (second eqn) box-alist))
                      (rhs (gen-answer (third eqn) box-alist))
                      (ret (apply #'nconc (mapcar (lambda (x)
                                                    (mapcar (lambda (y)
                                                              (list (first eqn) x y))
                                                            rhs))
                                                  lhs))))
                 ;; For a box of only numbers the user can either type
                 ;; the numbers and op or the evaluation of them.
                 (when (and (listp lhs)
                          (= (length lhs) 1)
                          (numberp (first lhs))
                          (listp rhs)
                          (= (length rhs) 1)
                          (numberp (first rhs)))
                     (push (funcall (first eqn) (first lhs) (first rhs))
                           ret))
                 ret))
               (t (error "huh2?")))))))


;;; The Operations

(defun safe-div (n d)
  "If D is 0 then return 0 to avoid a division by zero exception."
  (if (zerop d)
      0
    (/ n d)))

(defun verbose-minus-hint (n1 op n2 a)
  (labels ((pos (n) (>= n 0))
           (neg (n) (< n 0)))
    (ecase op
      (- ;;         (cond ((or (and (pos n1) (pos n2))
;;                    (and (neg n1) (pos n2)))
        (fmt "Change subtracting to adding the opposite (~a ~a ~a => ~a ~a ~a), then:~%~a"
             n1 op n2 n1 '+ (- n2)
             (verbose-minus-hint n1 '+ (- n2) a)))
;;               ((or (and (pos n1) (neg n2))
;;                    (and (neg n1) (neg n2)))
;;                (fmt "Change subtracting to adding the opposite."
;;                     ;; (fmt "~a ~a ~a = ~a ~a ~a = ~a" n1 op n2 n1 '+ (- n2) a)
;;                     ))
      (+
      (cond ((and (pos n1) (pos n2))
             (fmt "Add the numbers. A Positive number plus a positive number is positive."))
            ((and (neg n1) (neg n2))
             (fmt "First, add the absolute values: ~d + ~d~%The answer is negative because a negative plus a negative is negative." (abs n1) (abs n2)))
            (t
             (fmt "Subtract the two numbers' absolute values: ~d - ~d~%The answer has the sign of the term with the greatest absolute value." (max (abs n1) (abs n2)) (min (abs n1) (abs n2))))))
      (* (cond ((and (pos n1) (pos n2))
                (fmt "A positive times a positive is positive."))
               ((and (neg n1) (neg n2))
                (fmt "A negative times a negative is positive."))
               ((neg n1)
                (fmt "A negative times a positive is negative."))
               (t (fmt "A positive times a negative is negative."))))
      (/ (cond ((and (pos n1) (pos n2))
                (fmt "A positive divided by a positive is positive."))
               ((and (neg n1) (neg n2))
                (fmt "A negative divided by a negative is positive."))
               ((and (neg n1) (pos n2))
                (fmt "A negative divided by a positive is negative."))
               ((neg n1)
                (fmt "A negative divided by a positive is negative."))
               (t
                (fmt "A positive divided by a negative is negative.")))))))


(defoperation evaluate #\e evaluate #\e (symbols nth ((b1) (b1) (b1))
                                    ((:matches (:arg :op (:matches (+ - * /) :name op)
                                                     :arg1 (:matches :number :name n1)
                                                     :arg2 (:matches :number :name n2))
                                               :name b1)
                                     (:matches (:arg :op (:matches (*) :name op)
                                                     :arg1 (:matches 0 :name n1)
                                                     :arg2 (:matches :any :name n2))
                                               :name b1)                                     
                                     (:matches (:number :value (:matches :number :name n1))
                                               :name b1)))
  (with-matches (b1 op n1 n2) symbols
    (let* ((p (parent b1))
           (a  (ecase nth
                 (0 (funcall (if (eq op '/) 'safe-div op) n1 n2))
                 (1 0)
                 (2 n1)))
           (answer (make-answer-set)))
      ;; Hints
      (ecase nth
        (0 (if (find :verbose-minus (hint-types (tutor-problem-set *tutor*)))
               (set-both-hints (verbose-minus-hint n1 op n2 a))
             (set-both-hints (fmt "~a ~a ~a = ~a" n1 op n2 a))))
        (1 (set-both-hints (if (eq op '/) 
                               "0 divided by anything is 0."
                             "0 times anything is 0."))
           ;; If we remove them here then the graph changes when they
           ;; hit submit. we want these parents to get removed after
           ;; they submit.
           (setf (tutor-submit-post-fn *tutor*) (lambda () (mapc (lambda (b) (remove-box-parent b b1)) (only-boxes (children b1))))))
        (2 (set-hint "Copy the value according to the arrow." "Substitute X for its value")))
      (when p
        (if (typep p 'arg-box)
            (let (arg)
              (if (eq (arg-value (box-arg1 p)) b1)
                  (progn
                    (setf arg (box-arg1 p))
                    (add-answerf answer p (list (list (arg-value (box-op p)) a (arg-value (box-arg2 p))))))
                (progn
                  (setf arg (box-arg2 p))
                  (add-answerf answer p (list (list (arg-value (box-op p)) (arg-value (box-arg1 p)) a)))))
              ;; 	       (dformat "e argbox? ~a~%" arg)
              (setf (arg-answer arg) a
                    (arg-editable arg) t
                    (arg-user-answer arg) nil)
              (do-submit answer))
          ;; Answer box
          (progn
            ;; 	     (dformat "e answer? ~a~%" (arg-value (answer p)))
            (add-answerf answer p (list (list a)))
            (setf (arg-answer (answer p)) a
                  (arg-user-answer (answer p)) nil
                  (arg-editable (answer p)) t)
            (do-submit answer)))))))
  

(defun operator-name (op)
  (case op
    (+ "add")
    (* "multiply")
    (/ "divide")
    (- "subtract")))

(defoperation invert #\i unwind #\u (symbols nth ((b1 b2))
                                  ((:matches (:equals
                                              :arg1 (:matches :number :name n1)
                                              :arg2 (:matches (:arg :op (:matches (* / - +) :name op)
                                                                    :arg1 (:matches :box :name unknown)
                                                                    :arg2 (:matches :number :name x)
                                                                    :permute t)
                                                              :name b2)
                                              :permute t)
                                             :name b1)))
  ;;                                 (= @n (op @x ?q b2) b1) (= @n (op ?q @x b2) b1)
  ;;                                 (= (op @x ?q b2) @n b1) (= (op ?q @x b2) @n b1))
  ;;   (multiple-value-bind (arg1 arg2) (box-which-arg b1 (lambda (arg) (eq (arg-value arg) b2)))
  ;; We need to update ?q's parents
  ;;(declare (ignore nth))
  (with-matches (unknown n1 x op b1 b2) symbols
    (let ((x-on-left (eq unknown (arg-value (box-arg1 b2)))))
      (remove-box-parent unknown b2)
      (add-box-parent unknown b1)
      (setf (arg-value (box-arg1 b1)) unknown
            (arg-value (box-arg2 b1)) (make-box '* 0 0 b1 '* 0 0))
      (let ((answer (make-answer-set))
            (a (arg-value (box-arg2 b1))))
        ;;(dformat "op: ~a~%" op)
        (case op
          (+ (add-answerf answer a (op-answers (invert-op op) n1 x))
             (set-arg-box-answer a (invert-op op) n1 x)
             (set-both-hints (fmt "The result of adding ~a to something is ~a.
To find the value of that something, subtract ~a from ~a."
                       x n1 x n1)))
          (* (add-answerf answer a (op-answers (invert-op op) n1 x))
             (set-arg-box-answer a (invert-op op) n1 x)
             (set-both-hints (fmt "The result of multiplying ~a by something is ~a.
To find the value of that something, divide ~a by ~a."
                       x n1 n1 x)))
          (- (if x-on-left
                 (progn
                   (add-answerf answer a (op-answers (invert-op op) n1 x))
                   (set-arg-box-answer a (invert-op op) n1 x)
                   (set-both-hints (fmt "The result of subtracting ~a from something is ~a.
To find out the value of that something, add ~a to ~a."
                             x n1 x n1)))
               (progn
                 (add-answerf answer a (op-answers '- x n1))
                 (set-arg-box-answer a '- x n1)
                 (set-both-hints (fmt "The result of subtracting something from ~a is ~a.
To find out the value of that something, subtract ~a from ~a."
                           x n1 n1 x)))))
          (/ (if x-on-left
                 (progn
                   (add-answerf answer a (op-answers (invert-op op) n1 x))
                   (set-arg-box-answer a (invert-op op) n1 x)
                   (set-both-hints (fmt "The result of dividing ~a into something is ~a.
To find out the value of that something, multiply ~a by ~a."
                             x n1 n1 x)))
               (progn
                 (add-answerf answer a (op-answers '/ x n1))
                 (set-arg-box-answer a '/ x n1)
                 (set-both-hints (fmt "The result of dividing something into ~a is ~a.
To find out the value of that something, divide ~a into ~a."
                           x n1 n1 x))))))
        ;; 	 (dformat "answer: ~s~%" answer)
        (do-submit answer)))))

(defoperation reorder #\r distribute #\d (symbols nth ((b1 b2)
                                       (b1 b2))
                                  ((:matches (:arg :op (:matches (*) :name op1)
                                                   :arg1 (:matches :number :name n1)
                                                   :arg2 (:matches (:arg :op (:matches (+ -) :name op2)
                                                                         :arg1 (:matches :number :name n2)
                                                                         :arg2 (:matches :box :name box)
                                                                         :permute t)
                                                                   :name b2)
                                                   :permute t)
                                             :name b1)
                                   (:matches (:arg :op (:matches (/) :name op1)
                                                   :arg2 (:matches :number :name n1)
                                                   :arg1 (:matches (:arg :op (:matches (+ -) :name op2)
                                                                         :arg1 (:matches :number :name n2)
                                                                         :arg2 (:matches :box :name box)
                                                                         :permute t)
                                                                   :name b2))
                                             :name b1)))
  ;;(declare (ignore nth))
  (with-matches (op1 op2 n1 n2 box b1 b2) symbols
    (let ((parent (first (box-parents b1)))
          (carg (first (box-containers b1)))
          (left (box-left-child-p box b2))
          nb answers1
          ;; answers2 handles the case where 4*(X-5) => 4*X + 4*-5
          (answers2))
      (remove-box-parent box b2)
      (cond (left
             (multiple-value-setq (nb answers1) (new-subtree `(,op2 (,op1 ,n1 ,box) (,op1 ,n1 ,n2)) parent))
             ;; Our wart that allows 4*(X-5) => 4*X + 4*-5 to be a correct answer
             (when (eq op2 '-)
               (setf answers2 (make-answer-set))
               (let ((a1 (arg-value (box-arg1 nb)))
                     (a2 (arg-value (box-arg2 nb))))
                 (add-answerf answers2 nb (op-answers '+ a1 a2))
                 (add-answerf answers2 a1 (op-answers op1 n1 box))
                 (add-answerf answers2 a2 (op-answers op1 n1 (- n2)))))
             (dformat "ANSWER: ~s~%" answers2)
             (set-hint (fmt "Both operands in the ~a operator box are ~a by ~a."
                       op2 (if (eq op1 '*) "multiplied" "divided") n1)
                       (fmt "Both operands inside the parenthesis are ~a by ~a."
                            (if (eq op1 '*) "multiplied" "divided") n1)))
            (t
             (multiple-value-setq (nb answers1) (new-subtree `(,op2 (,op1 ,n1 ,n2) (,op1 ,n1 ,box)) parent))
             ;; Our wart that allows 4*(5-X) => 4*5 + -4*X to be a correct answer
             (when (eq op2 '-)
               (setf answers2 (make-answer-set))
               (let ((a1 (arg-value (box-arg1 nb)))
                     (a2 (arg-value (box-arg2 nb))))
                 (add-answerf answers2 nb (op-answers '+ a1 a2))
                 (add-answerf answers2 a1 (op-answers op1 (- n1) box))
                 (add-answerf answers2 a2 (op-answers op1 n1 n2))))
             (set-hint (fmt "Both operands in the ~a operator box are ~a by ~a."
                            op2 (if (eq op1 '*) "multiplied" "divided") n1)
                       (fmt "Both operands inside the parenthesis are ~a by ~a."
                            (if (eq op1 '*) "multiplied" "divided") n1))))
      (setf (arg-value carg) nb)
      ;; don't add answer2 if there is no second answer set
      (if answers2
          (do-submit answers1 answers2)
        (do-submit answers1)))))

(defun op-past-tense-to (op &optional object)
  (let ((mid (if object (format nil " ~a " object) " ")))
    (ecase op
      (+ (format nil "added~ato" mid))
      (- (format nil "subtracted~afrom" mid))
      (* (format nil "multiplied~ato" mid))
      (/ (format nil "divided~aby" mid)))))

(defun op-do-to (op &optional object)
  (let ((mid (if object (format nil " ~a " object) " ")))
    (ecase op
      (+ (format nil "add~ato" mid))
      (- (format nil "subtract~afrom" mid))
      (* (format nil "multiply~ato" mid))
      (/ (format nil "divide~aby" mid)))))


(defun op-do (op)
  (ecase op
    (+ "add")
    (- "subtract")
    (* "multiply")
    (/ "divide")))

(defun editable-boxes (root)
  "Return a list of boxes with editable components in them"
  (let (acc)
    (labels ((doit (box)
                   (typecase box
                     (arg-box
                      (when (or (arg-editable (box-arg1 box))
                                (arg-editable (box-arg2 box))
                                (arg-editable (box-op box)))
                        (push box acc)))
                     (answer-box
                      (when (arg-editable (answer box))
                        (push box acc))))
                   (mapc #'doit (only-boxes (children box)))))
      (doit root)
      acc)))

(defun left-most-box (b box)
  (< (box-x b) (box-x box)))
      ;;(< (box-y b) (box-y box))))

(defun right-most-box (b box)
  (> (box-x b) (box-x box)))
      ;;(< (box-y b) (box-y box))))

(defun bottom-most-box (b box)
  (> (box-y b) (box-y box)))

(defun top-most-box (b box)
  (< (box-y b) (box-y box)))

(defun x-most-box (boxes fn)
  (let ((box (car boxes)))
    (loop for b in boxes
          do (when (funcall fn b box)
               (setf box b)))
    box))

(defun two-at-the-top (boxes)
  "Return T if two boxes are at the top and at the same height."
  (let* ((top1 (x-most-box boxes 'top-most-box))
         (top2 (x-most-box (remove top1 boxes) 'top-most-box)))
    (= (box-y top1) (box-y top2))))

(defun click-the-box (b &optional loc)
  (setf loc (if loc 
                (concatenate 'string loc " ")
              ""))
  (typecase b
    (answer-box
     (format nil "Click the ~agreen box and type ~a." loc (arg-answer (answer b))))
    (arg-box
     (cond ((and (arg-editable (box-arg1 b))
                 (arg-editable (box-arg2 b)))
            (format nil "Click the ~agreen box and type ~a ~a ~a."
                    loc
                    (arg-answer (box-arg1 b))
                    (arg-answer (box-op b))
                    (arg-answer (box-arg2 b))))
           ((and (arg-editable (box-arg1 b))
                 (arg-editable (box-op b)))
            (format nil "Click the ~agreen box and type ~a ~a."
                    loc
                    (arg-answer (box-arg1 b))
                    (arg-answer (box-op b))))
           ((arg-editable (box-arg1 b))
            (format nil "Click the ~agreen box and type ~a." loc (arg-answer (box-arg1 b))))
           ((and (arg-editable (box-arg2 b))
                 (arg-editable (box-op b)))
            (format nil "Click the ~agreen box and type ~a ~a."
                    loc
                    (arg-answer (box-op b))
                    (arg-answer (box-arg2 b))))
           ((arg-editable (box-arg2 b))
            (format nil "Click the ~agreen box and type ~a." loc (arg-answer (box-arg2 b))))
           ((arg-editable (box-op b))
            (format nil "Click the ~agreen box and type ~a." loc (arg-answer (box-op b))))
           (t (error "didn't think of that!!"))))))

(defun click-instruction-text ()
  (if (tutor1-p)
      (tutor1-click-instruction-text)
    (tutor2-click-instruction-text)))

(defun tutor1-click-instruction-text ()
  "Return text that describes which boxes to click and what what to put in each one."
  (let ((boxes (editable-boxes (tutor-eqn-root *tutor*))))
    (ecase (length boxes)
      (0 nil)
      (1 (click-the-box (car boxes)))
      (2 (let ((bot (x-most-box boxes 'bottom-most-box))
               (top (x-most-box boxes (complement 'bottom-most-box))))
           (format nil "~a~%~a" 
                   (click-the-box top "top")
                   (click-the-box bot "bottom"))))
      (3 (if (two-at-the-top boxes)
             (let* ((bot (x-most-box boxes 'bottom-most-box))
                    (left (x-most-box boxes 'left-most-box))
                    (right (car (delete bot (remove left boxes)))))
               (format nil "~a~%~a~%~a" 
                       (click-the-box left "left")
                       (click-the-box right "right")
                       (click-the-box bot "bottom")))
           (let* ((bot (x-most-box boxes 'bottom-most-box))
                  (top (x-most-box boxes 'top-most-box))
                  (mid (car (delete bot (remove top boxes)))))
             (format nil "~a~%~a~%~a" 
                     (click-the-box top "top")
                     (click-the-box mid "middle")
                     (click-the-box bot "bottom")))))
      (4 (let* ((bot (x-most-box boxes 'bottom-most-box))
                (top (x-most-box boxes 'top-most-box))
                (right (x-most-box boxes 'right-most-box))
                (mid (car (delete bot (delete right (remove top boxes))))))
           (format nil "~a~%~a~%~a~%~a" 
                   (click-the-box top "left")
                   (click-the-box right "right")
                   (click-the-box mid "middle")
                   (click-the-box bot "bottom")))))))

(defun parent-list (box)
  (let ((p (parent box)))
    (unless (null p)
      (cons p (parent-list p)))))

(defun tutor2-gb-left-of (b1 b2)
  (let* ((pl1 (cons b1 (parent-list b1)))
         (pl2 (cons b2 (parent-list b2)))
         (common-parent (loop for a in pl1
                              until (find a pl2)
                              finally (return a))))
;;    (values pl1 pl2 common-parent)))
    (or (find-tree (box-left common-parent) (lambda (b) (eq b b1)))
        (and (eq b1 common-parent)
             (find-tree (box-right common-parent)
                        (lambda (b) (eq b b2)))))))

(defun tutor2-click-instruction-text ()
  "Return text that describes which boxes to click and what what to put in each one."
  (let ((boxes (sort (editable-boxes (tutor-eqn-root *tutor*))
                     'tutor2-gb-left-of)))
    (case (length boxes)
      (0 nil)
      (1 (click-the-box (car boxes)))
      (t (format nil "~a~%~{~a~^~%~}"
                 (click-the-box (first boxes) "left most")
                 (loop for i in (rest boxes)
                       collect (click-the-box i "next")))))))


;; (defoperation minus #\m minus #\m (symbols nth ((b1) (b1) (b1))
;;                                            ((:matches (:arg :arg1 (:matches :any :name b2)
;;                                                             :op (:matches (-) :name op)
;;                                                             :arg2 (:matches :number :name b3))
;;                                                       :name b1)
;;                                             (:matches (:arg :arg1 (:matches :any :name b2)
;;                                                             :op (:matches (-) :name op)
;;                                                             :arg2 (:matches (:x) :name b3))
;;                                                       :name b1)
;;                                             (:matches (:arg :arg1 (:matches :any :name b2)
;;                                                             :op (:matches (-) :name op)
;;                                                             :arg2 (:matches :box :name b3))
;;                                                       :name b1)))
;;   (with-matches (b1 b2 b3) symbols
;;     (let ((parent (first (box-parents b1)))
;;           (carg (first (box-containers b1))))
;;       (when (typep b3 'box)
;;         (remove-box-parent b3 b1))
;;       (when (typep b2 'box)
;;         (remove-box-parent b2 b1))
;;       (if (numberp b3)
;;           (multiple-value-bind (nb answers) (new-subtree `(+ ,b2 ,(- b3)) parent)
;;             (set-both-hints "Subtracting is the same as adding a negative.")
;;             (setf (arg-value carg) nb)
;;             (do-submit answers))
;;         (multiple-value-bind (nb answers) (new-subtree `(+ ,b2 (* -1 ,b3)) parent)
;;           (set-both-hints "Subtracting is the same as adding a negative.")
;;           (setf (arg-value carg) nb)
;;           (do-submit answers))))))

(defun box-or-var-text (box &optional particular)
  (if (typep box 'x-box)
      (format nil "~a variable" (if particular "the" "a"))
    (format nil "~a expression" (if particular "the" "an"))))

(defoperation combine #\c collect #\c (symbols nth ((b1 b2) (b1 b2) (b1 b2 b3) (b1 b2) (b1 b2 b3) (b1 b2 b3) (b1 b2 b3 b4) (b1 b2 b3 bx) (b1 b2 b3 b4 b5))
                                               ((:matches (:arg :op (:matches (+ -) :name op1)
                                                                :arg1 (:matches (:arg :op (:matches (+ -) :name op2)
                                                                                      :arg1 (:matches :number :name n2)
                                                                                      :arg2 (:matches :box :name box)
                                                                                      :permute t)
                                                                                :name b2)
                                                                :arg2 (:matches :number :name n1)
                                                                :permute t)
                                                          :name b1)
                                                (:matches (:arg :op (:matches (* /) :name op1)
                                                                :arg1 (:matches (:arg :op (:matches (* /) :name op2)
                                                                                      :arg1 (:matches :number :name n2)
                                                                                      :arg2 (:matches :box :name box)
                                                                                      :permute t)
                                                                                :name b2)
                                                                :arg2 (:matches :number :name n1)
                                                                :permute t)
                                                          :name b1)
                                                ;; combine unknown starts here
                                                (:matches (:arg :op (:matches (+ -) :name op1)
                                                                :arg1 (:matches (:arg :op (:matches (*) :name op2)
                                                                                      :arg1 (:matches (:x) :name x)
                                                                                      :arg2 (:matches :number :name n1)
                                                                                      :permute t)
                                                                                :name b2)
                                                                :arg2 (:matches (:arg :op (:matches (*) :name op2)
                                                                                      :arg1 (:matches (:x) :name x)
                                                                                      :arg2 (:matches :number :name n2)
                                                                                      :permute t)
                                                                                :name b3))
                                                          :name b1)
                                                ;; the original 1x
                                                (:matches (:arg :op (:matches (+ -) :name op1)
                                                                :arg1 (:matches (:arg :op (:matches (*) :name op2)
                                                                                      :arg1 (:matches (:x) :name x)
                                                                                      :arg2 (:matches :number :name n1)
                                                                                      :permute t)
                                                                                :name b2)
                                                                :arg2 (:matches (:x) :name x))
                                                          :name b1)
                                                ;; the other 1x
                                                (:matches (:arg :op (:matches (+ -) :name op1)
                                                                :arg1 (:matches (:arg :op (:matches (+ -) :name op2)
                                                                                      :arg1 (:matches (:x) :name x)
                                                                                      :arg2 (:matches :number :name n1)
                                                                                      :permute t)
                                                                                :name b2)
                                                                :arg2 (:matches (:arg :op (:matches (*) :name op3)
                                                                                      :arg1 (:matches (:x) :name x)
                                                                                      :arg2 (:matches :number :name n2)
                                                                                      :permute t)
                                                                                :name b3)
                                                                :permute t)
                                                          :name b1)
                                                ;; Yet another 1x
                                                (:matches (:arg :op (:matches (+ -) :name op1)
                                                                :arg1 (:matches (:arg :op (:matches (+ -) :name op2)
                                                                                      :arg1 (:matches (:arg :op (:matches (*) :name op3)
                                                                                                            :arg1 (:matches :number :name n1)
                                                                                                            :arg2 (:matches (:x) :name x)
                                                                                                            :permute t)
                                                                                                      :name b2)
                                                                                      :arg2 (:matches :number :name n2)
                                                                                      :permute t)
                                                                                :name b3)
                                                                :arg2 (:matches (:x) :name x)
                                                                :permute t)
                                                          :name b1)
                                                ;; YAYA1X 
                                                (:matches (:arg :op (:matches (+ -) :name op1)
                                                                :arg1 (:matches (:arg :op (:matches (+ -) :name op2)
                                                                                      :arg1 (:matches (:arg :op (:matches (*) :name op3)
                                                                                                            :arg1 (:matches :number :name n1)
                                                                                                            :arg2 (:matches (:x) :name x)
                                                                                                            :permute t)
                                                                                                      :name b2)
                                                                                      :arg2 (:matches :number :name n2)
                                                                                      :permute t)
                                                                                :name b3)
                                                                :arg2 (:matches (:arg :op (:matches (+ -) :name op3)
                                                                                      :arg1 (:matches (:x) :name x)
                                                                                      :arg2 (:matches :number :name n3))
                                                                                :name b4)
                                                                :permute nil)
                                                          :name b1)
                                                ;; nearly nasty
                                                (:matches (:arg :op (:matches (+ -) :name op1)
                                                                :arg1 (:matches (:arg :op (:matches (+ -) :name op3)
                                                                                      :arg1 (:matches (:arg :op (:matches (*) :name op2)
                                                                                                            :arg1 (:matches (:x) :name x)
                                                                                                            :arg2 (:matches :number :name n1)
                                                                                                            :permute t)
                                                                                                      :name b3)
                                                                                      :arg2 (:matches :number :name nx)
                                                                                      :permute t)
                                                                                :name bx)
                                                                :arg2 (:matches (:arg :op (:matches (*) :name op2)
                                                                                      :arg1 (:matches (:x) :name x)
                                                                                      :arg2 (:matches :number :name n2)
                                                                                      :permute t) :name b2)
                                                                :permute t)
                                                          :name b1)
                                                ;; The nastiest combine unknown rule
                                                (:matches (:arg :op (:matches (+ -) :name op1)
                                                                :arg1 (:matches (:arg :op (:matches (+ -) :name op2)
                                                                                      :arg1 (:matches (:arg :op (:matches (*) :name op3)
                                                                                                            :arg1 (:matches (:x) :name x)
                                                                                                            :arg2 (:matches :number :name n1)
                                                                                                            :permute t)
                                                                                                      :name b2)
                                                                                      :arg2 (:matches :number :name n2)
                                                                                      :permute t)
                                                                                :name b3)
                                                                :arg2 (:matches (:arg :op (:matches (+ -) :name op4)
                                                                                      :arg1 (:matches (:arg :op (:matches (*) :name op5)
                                                                                                            :arg1 (:matches (:x) :name x)
                                                                                                            :arg2 (:matches :number :name n3)
                                                                                                            :permute t)
                                                                                                      :name b4)
                                                                                      :arg2 (:matches :number :name n4)
                                                                                      :permute t)
                                                                                :name b5))
                                                          :name b1)))
  (if (<= nth 1)
      ;; combine numbers
      (with-matches (op1 op2 b1 b2 n1 n2 box) symbols
        (let ((parent (first (box-parents b1)))
              (carg (first (box-containers b1)))
              nb answers)
          (cond ((and (eq op1 op2)
                      (find op1 '(+ *)))
                 (let ((op-by (if (eq op1 '+) "added to" "multiplied by"))
                       (opname (if (eq op1 '+) "add" "multiply")))
                   (if (eq (arg-value (box-arg1 b1)) b2)
                       ;; (setup-answers answers b1 box box-arg2 box-arg1 op1 op1 n2 n1)
                       (progn
                         ;; (+ (+ X N2) N1)
                         (multiple-value-setq (nb answers) (new-subtree `(,op1 (,op1 ,n2 ,n1) ,box) parent))
                         (set-hint (fmt "First ~a is ~a a box then ~a is ~a that.~%So ~a ~a and ~a first, then ~a the box."
                                        n2 op-by n1 op-by opname n2 n1 opname)
                                   (fmt "First ~a is ~a ~a then ~a is ~a that.~%So ~a ~a and ~a first, then ~a the box."
                                        n2 op-by (box-or-var-text box) n1 op-by
                                        opname n2 n1 opname)))
                     ;; (setup-answers answers b1 box box-arg2 box-arg1 op1 op1 n1 n2)))
                     (progn
                       ;; (+ N1 (+ X N2))
                       (multiple-value-setq (nb answers) (new-subtree `(,op1 (,op1 ,n1 ,n2) ,box) parent))
                       (set-hint (fmt "First ~a is ~a a box then ~a is ~a that.~%So ~a ~a and ~a first, then ~a the box."
                                      n2 op-by n1 op-by opname n1 n2 opname)
                                 (fmt "First ~a is ~a ~a then ~a is ~a that.~%So ~a ~a and ~a first, then ~a ~a."
                                      n2 op-by (box-or-var-text box) n1 op-by
                                      opname n1 n2 opname (box-or-var-text box t)))))))
                ((and (eq op1 op2)
                      (find op1 '(- /)))
                 ;; We have to check where the number is
                 (cond ((and (eq (arg-value (box-arg1 b2)) box) 
                             (eq (arg-value (box-arg1 b1)) b2))
                        ;; (- (- X N2) N1)
                        ;;(setup-answers answers b1 box box-arg2 box-arg1 op1 op1 n1-invert n2))
                        (multiple-value-setq (nb answers) (new-subtree `(,op1 ,box (,(invert-op op1) ,n2 ,n1)) parent))
                        (set-hint (fmt "First ~a is ~a a box.~%Then ~a is ~a that.~%So ~a ~a and ~a and ~a it from the box."
                                       n2 (op-past-tense-to op1)
                                       n1 (op-past-tense-to op1)
                                       (op-do (invert-op op1)) n2 n1 (op-do op1))
                                  (fmt "First ~a is ~a ~a.~%Then ~a is ~a that.~%So ~a ~a and ~a and ~a it from ~a."
                                       n2 (op-past-tense-to op1) (box-or-var-text box)
                                       n1 (op-past-tense-to op1)
                                       (op-do (invert-op op1)) n2 n1 (op-do op1) (box-or-var-text box t))))
                       ((and (eq (arg-value (box-arg2 b2)) box)
                             (eq (arg-value (box-arg1 b1))  b2))
                        ;; (- (- N2 X) N1)
                        ;;(setup-answers answers b1 box box-arg2 box-arg1 op1 op1 n2 n1))
                        (multiple-value-setq (nb answers) (new-subtree `(,op1 (,op1 ,n2 ,n1) ,box) parent))
                        (set-hint (fmt "First a box is ~a ~a.~%Then ~a is ~a that.~%So ~a ~a from ~a first. Then ~a the box."
                                       (op-past-tense-to op1) n2
                                       n1 (op-past-tense-to op1)
                                       (op-do op1) n1 n2 (op-do op1))
                                  (fmt "First ~a is ~a ~a.~%Then ~a is ~a that.~%So ~a ~a from ~a first. Then ~a the box." 
                                       (box-or-var-text box) (op-past-tense-to op1) n2
                                       n1 (op-past-tense-to op1)
                                       (op-do op1) n1 n2 (op-do op1))))
                       ((and (eq (arg-value (box-arg1 b2)) box)
                             (eq (arg-value (box-arg2 b1)) b2))
                        ;; (- N1 (- X N2))
                        ;;(setup-answers answers b1 box box-arg2 box-arg1 op1 (invert-op op1) n1 n2))
                        (multiple-value-setq (nb answers) (new-subtree `(,op1 (- ,n1 ,(- n2)) ,box) parent))
                        (set-hint (fmt "Subtracting ~a is the same as adding ~a.~%First a box is added to ~a then subtracted from ~a.~%So ~a ~a and ~a, then ~a a box."
                                       n2 (- n2)
                                       (- n2) n1
                                       (op-do '-) n1 (- n2) (op-do op1))
                                  (fmt "Subtracting ~a is the same as adding ~a.~%First ~a is added to ~a then subtracted from ~a.~%So ~a ~a and ~a, then ~a ~a." 
                                       n2 (- n2)
                                       (box-or-var-text box) (- n2) n1
                                       (op-do '-) n1 (- n2) (op-do op1) (box-or-var-text box t))))
                       ((and (eq (arg-value (box-arg2 b2)) box)
                             (eq (arg-value (box-arg2 b1)) b2))
                        ;; (- N1 (- N2 X))
                        ;;(setup-answers answers b1 box box-arg2 box-arg1 op1 op1 n1 n2))
                        (multiple-value-setq (nb answers) (new-subtree `(,(invert-op op1) (,op1 ,n1 ,n2) ,box) parent))
                        (set-hint (fmt "~:(~a~) ~a from ~a, then ~a the unknown."
                                       (op-do op1) n2 n1 (op-do (invert-op op1)))
                                  (fmt "~:(~a~) ~a from ~a, then ~a ~a." (op-do op1) n2 n1 (op-do (invert-op op1)) (box-or-var-text box t))))
                       (t (error "didn't think of that"))))
                ;; Different OPs
                ((find op1 '(+ *))
                 (if (eq (arg-value (box-arg1 b2)) box)
                     ;;(setup-answers answers b1 box box-arg2 box-arg1 op2 op1 n1 n2)
                     (progn
                       ;; (+ N1 (- X N2))
                       (multiple-value-setq (nb answers) (new-subtree `(,op1 (,op2 ,n1 ,n2) ,box) parent))
                       (set-hint (fmt "First ~a is ~a a box then ~a is ~a that.~%So ~a ~a then ~a the box."
                                      n2 (op-past-tense-to op2) n1 (op-past-tense-to op1)
                                      (op-do-to op2 n2) n1 (op-do op1))
                                 (fmt "First ~a is ~a ~a then ~a is ~a that.~%So ~a ~a then ~a ~a."
                                      n2 (op-past-tense-to op2) (box-or-var-text box) n1 (op-past-tense-to op1)
                                      (op-do-to op2 n2) n1 (op-do op1) (box-or-var-text box t))))
                   ;;(setup-answers answers b1 box box-arg2 box-arg1 op2 op1 n2 n1)))
                   (progn
                     ;; (+ N1 (- N2 X))
                     (multiple-value-setq (nb answers) (new-subtree `(,op2 (,op1 ,n2 ,n1) ,box) parent))
                     (set-hint (fmt "First a box is ~a ~a then ~a is ~a that.~%So ~a ~a and ~a then ~a the box."
                                    (op-past-tense-to op2) n2 n1 (op-past-tense-to op1)
                                    (op-do op1) n1 n2 (op-do op2))
                               (fmt "First ~a is ~a ~a then ~a is ~a that.~%So ~a ~a and ~a then ~a ~a."
                                    (box-or-var-text box t) (op-past-tense-to op2) n2 n1 (op-past-tense-to op1)
                                    (op-do op1) n1 n2 (op-do op2) (box-or-var-text box t))))))
                ((find op1 '(- /))
                 (if (eq (arg-value (box-arg1 b1)) b2)
                     ;;(setup-answers answers b1 box box-arg2 box-arg1 op2 op1 n2 n1)
                     (progn
                       ;; (- (+ X N2) N1)
                       (multiple-value-setq (nb answers) (new-subtree `(,op2 ,box (,op1 ,n2 ,n1)) parent))
                       (set-hint (fmt "First ~a is ~a a box then ~a is ~a that.~%So ~a ~a then ~a the box."
                                      n2 (op-past-tense-to op2) n1 (op-past-tense-to op1)
                                      (op-do-to op1 n1) n2 (op-do-to op2 "it"))
                                 (fmt "First ~a is ~a ~a then ~a is ~a that.~%So ~a ~a then ~a ~a."
                                      n2 (op-past-tense-to op2) (box-or-var-text box) n1 (op-past-tense-to op1)
                                      (op-do-to op1 n1) n2 (op-do-to op2 "it") (box-or-var-text box t))))
                   ;;(setup-answers answers b1 box box-arg2 box-arg1 op1 op2 n1 n2)))
                   (progn
                     ;; (- (+ N2 X) N1)
                     (multiple-value-setq (nb answers) (new-subtree `(,op1 (,op1 ,n1 ,n2) ,box) parent))
                     (set-hint (fmt "Both a box and ~a are ~a ~a.~%So ~a ~a then ~a the box."
                                    n2 (op-past-tense-to op1) n1
                                    (op-do-to op1 n2) n1 (op-do op1))
                               (fmt "Both ~a and ~a are ~a ~a.~%So ~a ~a then ~a ~a."
                                    (box-or-var-text box t) n2 (op-past-tense-to op1) n1
                                    (op-do-to op1 n2) n1 (op-do op1) (box-or-var-text box t))))))
                (t (error "didn't think of that!")))
          (remove-box-parent box b2)
          (setf (arg-value carg) nb)
          (do-submit answers)))
    ;; !!!!!!!!!!!!!!!!!!
    ;; combine unknown  
    ;; !!!!!!!!!!!!!!!!!!
    (with-matches (x b1 b2 b3 b4 op1 op2 op3 op4 n1 n2 n3 n4 nx) symbols
      ;; b1 has 1 parent.
      (let ((parent (first (box-parents b1)))
            (carg (first (box-containers b1)))
            nb answers)
        (ecase nth
          (2
           ;; the first rule fired
           ;; (+ (* n1 X) (* n2 X))
           (remove-box-parent x b3)
           (remove-box-parent x b2)
           (multiple-value-setq (nb answers) (new-subtree `(* (,op1 ,n1 ,n2) ,x) parent))
           (if (eq op1 '-)
               (set-hint (fmt "~a unknowns are subtracted from ~a unknowns.~%So subtract ~a from ~a, then multiply by the unknown."
                              n2 n1 n2 n1)
                         (fmt "~a Xs are subtracted from ~a Xs.~%So subtract ~a from ~a, then multiply by X."
                              n2 n1 n2 n1))
             (set-hint (fmt "~a unknowns and ~a unknowns are added.~%So add ~a and ~a, then multiply by the unknown."
                            n1 n2 n1 n2)
                       (fmt "~a Xs and ~a Xs are added.~%So add ~a and ~a, then multiply by X."
                            n1 n2 n1 n2))))
          (3
           ;; the second rule fired
           ;; (+ (* n1 X) X)
           (remove-box-parent x b1)
           (remove-box-parent x b2)
           (multiple-value-setq (nb answers) (new-subtree `(* (,op1 ,n1 1) ,x) parent))
           (if (eq op1 '-)
               (set-hint (fmt "1 unknown is subtracted from ~a unknowns.~%So subtract 1 from ~a first. then multiply the unknown."
                              n1 n1)
                         (fmt "1 X is subtracted from ~a Xs.~%So subtract 1 from ~a first. then multiply by X."
                              n1 n1))
             (set-hint (fmt "~a unknowns and 1 unknown are added.~%So add ~a and 1 first. then multiply the unknown."
                            n1 n1)
                       (fmt "~a Xs and 1 X are added.~%So add ~a and 1 first. then multiply by X."
                            n1 n1))))
          (4
           (cond ((simple-match b1 '(- (* :n :x) (- :x :n)))
                  (multiple-value-setq (nb answers) (new-subtree `(- (* (- ,n2 1) ,x) ,(- n1)) parent))
                  (set-hint (fmt "The right side is subtracted from the left side.
1 unknown is subtracted from ~a unknowns.
~a is subtracted from the left side.
So subtract 1 from ~a. Then multiply the unknown.
Finally, subtract ~a."
                                 n2
                                 (- n1)
                                 n2
                                 (- n1)
                                 n2 '- 1)
                            (fmt "The right side is subtracted from the left side.
1 X is subtracted from ~a Xs.
~a is subtracted from the left side.
So subtract ~a and 1. Then multiply by X.
Finally subtract ~a."
                                 n2
                                 (- n1)
                                 n2
                                 (- n1)
                                 n2 '- 1)))
                 ((simple-match b1 '(- (* :n :x) (- :n :x)))
                  (multiple-value-setq (nb answers) (new-subtree `(- (* (- ,n2 -1) ,x) ,n1) parent))
                  (set-hint (fmt "The right side is subtracted from the left side.
-1 unknowns is subtracted from ~a unknowns.
~a is subtracted from the left side.
So subtract ~a and -1. Then multiply the unknown.
Finally subtract ~a."
                                 n2
                                 n1
                                 n2
                                 n1)
                            (fmt "The right side is subtracted from the left side.
-1 Xs is subtracted from ~a Xs.
~a is subtracted from the left side.
So subtract ~a and -1. Then multiply by X.
Finally subtract ~a."
                                 n2
                                 n1
                                 n2
                                 n1)))
                 ((simple-match b1 '(- (- :n :x) (* :n :x)))
                  (multiple-value-setq (nb answers) (new-subtree `(,op2 (* (,op1 1 ,n2) ,x) ,n1) parent))
                  (set-hint (fmt "1 unknown is subtracted from ~a unknowns. ~a is ~a 1 unknown.
So ~a 1 and ~a first. Then multiply the unknown.
Finally, ~a ~a."
                                  n2
                                  n1 (op-past-tense-to op2)
                                  (op-do op1) n2
                                  (op-do op2) n1)
                             (fmt "1 X is subtracted from ~a Xs. ~a is ~a 1 X.
So ~a 1 and ~a first. Then multiply by X.
Finally, ~a ~a."
                                  n2
                                  n1 (op-past-tense-to op2)
                                  (op-do op1) n2
                                  (op-do op2) n1)))
                 ((or (simple-match b1 '(+ (+ :n :x) (* :n :x)))
                      (simple-match b1 '(+ (+ :x :n) (* :n :x))))
                  (multiple-value-setq (nb answers) (new-subtree `(,op2 (* (,op1 1 ,n2) ,x) ,n1) parent))
                  (set-hint (fmt "1 unknown and ~a unknowns are added. ~a is ~a 1 unknown.
So ~a 1 and ~a first. Then multiply the unknown.
Finally, ~a ~a."
                                 n2
                                 n1 (op-past-tense-to op2)
                                 (op-do op1) n2
                                 (op-do op2) n1)
                            (fmt "1 X and ~a Xs are added. ~a is ~a 1 X.
So ~a 1 and ~a first. Then multiply by X.
Finally, ~a ~a."
                                 n2
                                 n1 (op-past-tense-to op2)
                                 (op-do op1) n2
                                 (op-do op2) n1)))
                 (t
                  (error "This case has not been accounted for 1.")))
           ;; Remove the parents at the end otherwise box-left-child-p won't work.
           (remove-box-parent x b2)
           (remove-box-parent x b3))
          (5
           ;; the YA 1x rule
           ;; currently ONLY handles this case:
           ;;(+ (- n2 (* n1 x)) x)
           (cond ((simple-match b1 '(+ (- :n (* :n :x)) :x))
                  (multiple-value-setq (nb answers) (new-subtree `(+ ,n2 (* (+ ,(- n1) 1) ,x)) parent))
                  (set-hint (fmt "Subtracting ~a unknowns is the same as adding ~a unknowns.
~a unknowns and 1 unknown are added
~a unknowns is added to ~a.
So add ~a and 1 first. Then multiply the unknown.
Finally add it to ~a."
                                 n1 (- n1)
                                 (- n1)
                                 (- n1) n2
                                 (- n1)
                                 n2)
                            (fmt "Subtracting ~a Xs is the same as adding ~a Xs.
~a Xs and 1 X are added
~a Xs is added to ~a.
So add ~a and 1 first. Then multiply by X.
Finally add it to ~a."
                                 n1 (- n1)
                                 (- n1)
                                 (- n1) n2
                                 (- n1)
                                 n2)))
                 (t
                  (error "thisi case has not been accounted for.")))
           ;; reparent at the end...
           (remove-box-parent x b2)
           (remove-box-parent x b1))
          (6
           (cond ((or (simple-match b1 '(- (+ (* :n :x) :n) (- :x :n)))
                      (simple-match b1 '(- (+ :n (* :n :x)) (- :x :n))))
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (- ,n1 1) ,x) (- ,n2 ,(- n3))) parent))
                  (set-hint (fmt "The right side is subtracted from the left side.
So subtract ~a from ~a, then multiply the unknown.
Finally, subtract ~a from ~a and add it to the rest."
                                 1 n1
                                 (- n3) n2)
                            (fmt "Subtract ~a from ~a, then multiply by X.
Finally, subtract ~a from ~a and add it to the rest."
                                 1 n1
                                 (- n3) n2)))
                 ((simple-match b1 '(- (- :n (* :n :x)) (- :x :n)))
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (- ,(- n1) 1) ,x) (- ,n2 ,(- n3))) parent))
                  (set-hint (fmt "The right side is subtracted from the left side.
So subtract ~a from ~a, then multiply the unknown.
Finally, subtract ~a from ~a and add it to the rest."
                                 1 (- n1)
                                 n3 n2)
                            (fmt "So subtract ~a from ~a, then multiply by X.
Finally, subtract ~a and from ~a and add it to the rest."
                                 1 (- n1)
                                 n3 n2)))
                 (t
                  (error "this case has not been accounted for 2")))
           (remove-box-parent x b2)
           (remove-box-parent x b4))
          (7
           (cond ((or (simple-match b1 '(+ (+ (* :n :x) :n) (* :n :x)))
                      (simple-match b1 '(+ (+ :n (* :n :x)) (* :n :x)))
                      (simple-match b1 '(+ (- (* :n :x) :n) (* :n :x))))
                  (dformat "SOLID '(+ (+ (* :n :x) :n) (* :n :x))~%'(+ (- (* :n :x) :n) (* :n :x))~%")
                  (multiple-value-setq (nb answers) (new-subtree `(,op3 (* (,op1 ,n1 ,n2) ,x) ,nx) parent))
                  (set-hint (fmt "~a unknowns and ~a unknowns are added.
~a is ~a the ~a unknowns.
So add ~a and ~a first. Then multiply the unknown.
Finally, ~a ~a."
                                 n1 n2
                                 nx (op-past-tense-to op3) n1
                                 n1 n2 (op-do op3) nx)
                            (fmt "Collect the ~a Xs and ~a Xs by
adding ~a and ~a and multiplying by X.
Finally, ~a ~a."
                                 n1 n2
                                 n1 n2 (op-do op3) nx)))
                 ((or (simple-match b1 '(- (- (* :n :x) :n) (* :n :x)))
                      (simple-match b1 '(- (+ (* :n :x) :n) (* :n :x)))
                      (simple-match b1 '(- (+ :n (* :n :x)) (* :n :x))))
                  (dformat "SOLID '(- (- (* :n :x) :n) (* :n :x))~%'(- (+ (* :n :x) :n) (* :n :x))~%")
                  (multiple-value-setq (nb answers) (new-subtree `(,op3 (* (,op1 ,n1 ,n2) ,x) ,nx) parent))
                  (set-hint (fmt "~a unknowns are subtracted from ~a unknowns.
~a is ~a the ~a unknowns.
So subtract ~a from ~a first. Then multiply the unknown.
Finally, ~a ~a."
                                 n2 n1
                                 nx (op-past-tense-to op3) n1
                                 n2 n1 (op-do op3) nx)
                            (fmt "~a Xs are subtracted from ~a Xs.
So subtract ~a from ~a first. Then multiply by X.
Finally, ~a ~a."
                                 n2 n1
                                 n2 n1 (op-do op3) nx)))
                 ((simple-match b1 '(+ (- :n (* :n :x)) (* :n :x)))
                  (dformat "SOLID '(+ (- :n (* :n :x)) (* :n :x))")
                  (multiple-value-setq (nb answers) (new-subtree `(+ ,nx (* (,op1 ,(- n1) ,n2) ,x)) parent))
                  (set-hint (fmt "Subtracting ~a unknowns is the same as adding ~a unknowns.
~a unknowns and ~a unknowns are added.
the ~a unknowns are added to ~a.
So add ~a and ~a first. Then multiply the unknown.
Finally, ~a ~a."
                                 n1 (- n1)
                                 (- n1) n2
                                 (- n1) nx
                                 (- n1) n2
                                 (op-do-to '+ "it") nx)
                            (fmt "Subtracting ~a Xs is the same as adding ~a Xs.
Collect the ~a Xs and ~a Xs by
adding ~a and ~a and multiplying by X.
Finally, ~a ~a."
                                 n1 (- n1)
                                 (- n1) n2
                                 (- n1) n2
                                 (op-do-to '+ "it") nx)))
                 ((or (simple-match b1 '(+ (* :n :x) (- (* :n :x) :n)))
                      (simple-match b1 '(+ (* :n :x) (+ (* :n :x) :n)))
                      (simple-match b1 '(+ (* :n :x) (+ :n (* :n :x)))))
                  (dformat "SOLID '(+ (* :n :x) (- (* :n :x) :n))~%'(+ (* :n :x) (+ (* :n :x) :n))~%")
                  (multiple-value-setq (nb answers) (new-subtree `(,op3 (* (,op1 ,n2 ,n1) ,x) ,nx) parent))
                  (set-hint (fmt "~a unknowns and ~a unknowns are added.
~a is ~a the ~a unknowns.
So add ~a and ~a first. Then multiply the unknown.
Finally, ~a ~a."
                                 n2 n1
                                 nx (op-past-tense-to op3) n1
                                 n2 n1 (op-do op3) nx)
                            (fmt "Collect the ~a Xs and ~a Xs by
adding ~a and ~a and multiplying by X.
Finally, ~a ~a."
                                 n2 n1
                                 n2 n1
                                 (op-do op3) nx)))
                 ((simple-match b1 '(- (* :n :x) (- :n (* :n :x))))
                  (dformat "SOLID '(- (* :n :x) (- :n (* :n :x)))~%")
                  (multiple-value-setq (nb answers) (new-subtree `(- (* (- ,n2 ,(- n1)) ,x) ,nx) parent))
                  (set-hint (fmt "The right side is subtracted from the left side.
Subtracting ~a unknowns is the same as adding ~a unknowns.
So subtract ~a from ~a, then multiply the unknown.
Finally, subtract ~a."
                                 n1 (- n1)
                                 (- n1) n2
                                 nx)
                            (fmt "Subtracting ~a Xs is the same as adding ~a Xs.
So subtract ~a from ~a, then multiply by X.
Finally, subtract ~a."
                                 n1 (- n1)
                                 (- n1) n2
                                 nx)))
                 ((or (simple-match b1 '(- (* :n :x) (+ :n (* :n :x))))
                      (simple-match b1 '(- (* :n :x) (+ (* :n :x) :n))))
                  (dformat "SOLID '(- (* :n :x) (+ :n (* :n :x)))~%")
                  (multiple-value-setq (nb answers) (new-subtree `(- (* (- ,n2 ,n1) ,x) ,nx) parent))
                  (set-hint (fmt "The right side is subtracted from the left side.
So subtract ~a from ~a, then multiply the unknown.
Finally, subtract ~a."
                                 n1 n2
                                 nx)
                            (fmt "Subtract ~a from ~a, then multiply by X.
Finally, subtract ~a."
                                 n1 n2
                                 nx)))
                 ((simple-match b1 '(+ (* :n :x) (- :n (* :n :x))))
                  (dformat "'(+ (+ (* :n :x) :n) (* :n :x))")
                  (multiple-value-setq (nb answers) (new-subtree `(+ ,nx (* (,op1 ,n2 ,(- n1)) ,x)) parent))
                  (set-hint (fmt "Subtracting ~a unknowns is the same as adding ~a unknowns.
~a unknowns and ~a unknowns are added.
the ~a unknowns are added to ~a.
So add ~a to ~a first. Then multiply the unknown.
Finally, ~a ~a."
                                 n1 (- n1)
                                 n2 (- n1)
                                 (- n1) nx
                                 (- n1) n2
                                 (op-do-to '+ "it") nx)
                            (fmt "Subtracting ~a Xs is the same as adding ~a Xs.
Collect the ~a Xs and ~a Xs by
adding ~a and ~a and multiplying by X.
Finally, ~a ~a."
                                 n1 (- n1)
                                 n2 (- n1)
                                 n2 (- n1)
                                 (op-do-to '+ "it") nx)))
                 ((simple-match b1 '(- (* :n :x) (- (* :n :x) :n)))
                  (dformat "SOLID '(- (* :n :x) (- (* :n :x) :n))")
                  (multiple-value-setq (nb answers) (new-subtree `(- (* (- ,n2 ,n1) ,x) ,(- nx)) parent))
                  (set-hint (fmt "The right side is subtracted from the left side.
So subtract ~a from ~a, then multiply the unknown.
Finally, subtract ~a."
                                 n1 n2
                                 (- nx))
                            (fmt "Subtract ~a from ~a, then multiply by X.
Finally, subtract ~a."
                                 n1 n2
                                 (- nx))))
                 ((simple-match b1 '(- (- :n (* :n :x)) (* :n :x)))
                  (dformat "'(- (- :n (* :n :x)) (* :n :x))~%")
                  (multiple-value-setq (nb answers) (new-subtree `(+ ,nx (* (,op1 ,(- n1) ,n2) ,x)) parent))
                  (set-hint (fmt "Subtracting ~a unknowns is the same as adding ~a unknowns.
~a unknowns are subtracted from ~a unknowns.
the ~a unknowns are added to ~a.
So subtract ~a from ~a first. Then multiply the unknown.
Finally, ~a ~a."
                                 n1 (- n1)
                                 n2 (- n1)
                                 (- n1) nx
                                 n2 (- n1)
                                 (op-do-to '+ "it") nx)
                            (fmt "Subtracting ~a Xs is the same as adding ~a Xs.
~a Xs are subtracted from ~a Xs.
So subtract ~a from ~a first. Then multiply by X.
Finally, ~a ~a."
                                 n1 (- n1)
                                 n2 (- n1)
                                 (- n1) nx
                                 n2 (- n1)
                                 (op-do-to '+ "it") nx)))
                 (t
                  (error "forgot to handle this case.")))
           ;; remove the boxes at the end otherwise box-left-child-p won't work.
           (remove-box-parent x b3)
           (remove-box-parent x b2))
          (8
           ;; (+ (+ (* n1 X) n2) (+ (* n3 X) n4))
           (cond ((and (not (box-left-child-p b4))
                       (box-left-child-p b2)
                       (eq op1 '+)
                       (eq op2 '-)
                       (eq op4 '-))
                  (dformat "SOLID here1~%")
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (+ ,n1 ,(- n3)) ,x) (+ ,(- n2) ,n4)) parent))
                  (set-hint (fmt "Subtracting ~a unknowns is the same as adding ~a unknowns.
~a unknowns and ~a unknowns are added.
~a is added to ~a.
So add ~a to ~a first. Then multiply the unknown.
Finally, add ~a and ~a. and add it to the rest."
                                 n3 (- n3)
                                 n1 (- n3)
                                 (- n2) n4
                                 n1 (- n3)
                                 (- n2) n4)
                            (fmt "Subtracting ~a Xs is the same as adding ~a Xs.
Collect the ~a Xs and ~a Xs and
collect the ~a and ~a by
adding ~a to ~a then multiplying by X.
Finally, add ~a and ~a and add it to the rest."
                                 n3 (- n3)
                                 n1 (- n3)
                                 (- n2) n4
                                 n1 (- n3)
                                 (- n2) n4)))
                 ((and (box-left-child-p b4)
                       (eq op1 '+)
                       (eq op2 '-)
                       (eq op4 '-))
                  (dformat "SOLID here2~%")
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (+ ,n1 ,n3) ,x) (- ,(- n2) ,n4)) parent))
                  (set-hint (fmt "~a unknowns and ~a unknowns are added
~a is subtracted from ~a.
So add ~a to ~a first. Then multiply the unknown.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n1 n3
                                 n4 (- n2)
                                 n1 n3
                                 n4 (- n2))
                            (fmt "~a Xs and ~a Xs are added
~a is subtracted from ~a.
So add ~a to ~a first. Then multiply by X.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n1 n3
                                 n4 (- n2)
                                 n1 n3
                                 n4 (- n2))))
                 ((and (not (box-left-child-p b4))
                       (eq op1 '+)
                       (eq op2 '-)
                       (eq op4 '-))
                  (dformat "here2-2~%")
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (- ,n1 ,(- n3)) ,x) (+ ,n2 ,n4)) parent))
                  (set-hint (fmt "Subtracting ~a unknowns is the same as adding ~a unknowns.
~a unknowns and ~a unknowns are added.
~a is added to ~a.
So add ~a to ~a first. Then multiply the unknown.
Finally, add ~a and ~a. and add it to the rest."
                                 n3 (- n3)
                                 n1 (- n3)
                                 n2 n4
                                 n1 (- n3)
                                 n2 n4)
                            (fmt "Subtracting ~a Xs is the same as adding ~a Xs.
~a Xs and ~a Xs are added.
~a is added to ~a.
So add ~a to ~a first. Then multiply by X.
Finally, add ~a and ~a. and add it to the rest."
                                 n3 (- n3)
                                 n1 (- n3)
                                 n2 n4
                                 n1 (- n3)
                                 n2 n4)))
                 ((and (eq op1 '-)
                       (eq op2 '+)
                       (eq op4 '+))
                  (dformat "SOLID here3~%")
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (- ,n1 ,n3) ,x) (- ,n2 ,n4)) parent))
                  (set-hint (fmt "The right side is subtracted from the left side.
~a unknowns are subtracted from the left side and
~a is subtracted from the left side.
So subtract ~a from ~a first. Then multiply the unknown.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n3
                                 n4
                                 n3 n1
                                 n4 n2)
                            (fmt "The right side is subtracted from the left side.
~a Xs are subtracted from the left side and
~a is subtracted from the left side.
So subtract ~a from ~a first. Then multiply by X.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n3
                                 n4
                                 n3 n1
                                 n4 n2)))
                 ((and (not (box-left-child-p b4))
                       (box-left-child-p b2)
                       (eq op1 '-)
                       (eq op2 '-)
                       (eq op4 '-))
                  (dformat "SOLID here4~%")
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (- ,n1 ,(- n3)) ,x) (- ,(- n2) ,n4)) parent))
                  (set-hint (fmt "The right side is subtracted from the left side.
Subtracting ~a Xs is the same as adding ~a Xs.
~a unknowns are subtracted from ~a unknowns.
~a is subtracted from ~a.
So subtract ~a from ~a first. Then multiply the unknown.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n3 (- n3)
                                 (- n3) n1
                                 n4 (- n2)
                                 (- n3) n1
                                 n4 (- n2))
                            (fmt "Subtracting ~a Xs is the same as adding ~a Xs.
~a Xs are subtracted from the left side and
~a is subtracted from from the left side.
So subtract ~a from ~a first. Then multiply by X.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n3 (- n3)
                                 (- n3)
                                 n4
                                 (- n3) n1
                                 n4 (- n2))))
                 ((and (box-left-child-p b2)
                       (eq op1 '+)
                       (eq op2 '-)
                       (eq op4 '+))
                  (dformat "here5~%")
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (+ ,n1 ,n3) ,x) (+ ,(- n2) ,n4)) parent))
                  (set-hint (fmt "~a unknowns and ~a unknowns are added.
~a and ~a are added.
So add ~a and ~a first. Then multiply the unknown.
Finally, add ~a and ~a. and add it to the rest."
                                 n1 n3
                                 (- n2) n4
                                 n1 n3
                                 (- n2) n4)
                            (fmt "~a Xs and ~a Xs are added.
~a and ~a are added.
So add ~a and ~a first. Then multiply by X.
Finally, add ~a and ~a. and add it to the rest."
                                 n1 n3
                                 (- n2) n4
                                 n1 n3
                                 (- n2) n4)))
                 ((and (not (box-left-child-p b4))
                       (eq op1 '-)
                       (eq op2 '+)
                       (eq op4 '-))
                  (dformat "SOLID here6~%")
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (- ,n1 ,(- n3)) ,x) (- ,n2 ,n4)) parent))
                  (set-hint (fmt "The right side is subtracted from the left side.
Subtracting ~a Xs is the same as adding ~a Xs.
~a unknowns are subtracted from ~a unknowns.
~a is subtracted from ~a.
So subtract ~a from ~a first. Then multiply the unknown.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n3 (- n3)
                                 (- n3) n1
                                 n4 n2
                                 (- n3) n1
                                 n4 n2)
                            (fmt "
Subtracting ~a Xs is the same as adding ~a Xs.
~a Xs are subtracted from the left side and
~a is subtracted from the left side.
So subtract ~a from ~a first. Then multiply X.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n3 (- n3)
                                 (- n3)
                                 n4
                                 (- n3) n1
                                 n4 n2)))
                 ((and (eq op1 '-)
                       (eq op2 '+)
                       (eq op4 '-))
                  (dformat "SOLID here7~%")
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (- ,n1 ,n3) ,x) (- ,n2 ,(- n4))) parent))
                  (set-hint (fmt "The right side is subtracted from the left side.
~a unknowns are subtracted from ~a unknowns.
~a is subtracted from ~a.
So subtract ~a from ~a first. Then multiply the unknown.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n3 n1
                                 (- n4) n2
                                 n3 n1
                                 (- n4) n2)
                            (fmt "~a Xs are subtracted from ~a Xs.
~a is subtracted from ~a.
So subtract ~a from ~a first. Then multiply by X.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n3 n1
                                 (- n4) n2
                                 n3 n1
                                 (- n4) n2)))
                 ((and (box-left-child-p b4)
                       (box-left-child-p b2)
                       (eq op1 '-)
                       (eq op2 '-)
                       (eq op4 '-))
                  (dformat "here7-2~%")
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (- ,n1 ,n3) ,x) (- ,(- n2) ,(- n4))) parent))
                  (set-hint (fmt "The right side is subtracted from the left side.
~a unknowns are subtracted from ~a unknowns.
~a is subtracted from ~a.
So subtract ~a from ~a first. Then multiply the unknown.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n3 n1
                                 (- n4) (- n2)
                                 n3 n1
                                 (- n4) (- n2))
                            (fmt "~a Xs are subtracted from ~a Xs.
~a is subtracted from ~a.
So subtract ~a from ~a first. Then multiply by X.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n3 n1
                                 (- n4) (- n2)
                                 n3 n1
                                 (- n4) (- n2))))
                 ((and (eq op1 '+)
                       (eq op2 '+)
                       (eq op4 '+))
                  (dformat "SOLID here8~%")
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (+ ,n1 ,n3) ,x) (+ ,n2 ,n4)) parent))
                  (set-hint (fmt "~a unknowns and ~a unknowns are ~a.
~a and ~a are ~a.
So ~a ~a and ~a first. Then multiply the unknown.
Finally, ~a ~a and ~a. and add it to the rest."
                                 n1 n3 "added"
                                 n2 n4 "added"
                                 (op-do op1) n1 n3
                                 (op-do op1) n2 n4)
                            (fmt "Collect the ~a Xs and ~a Xs and
collect the ~a and ~a by
adding ~a and ~a and multiplying by X.
Finally, add ~a and ~a and add it to the rest."
                                 n1 n3
                                 n2 n4
                                 n1 n3
                                 n2 n4)))
                 ((and (box-left-child-p b4)
                       (eq op1 '+)
                       (eq op2 '+)
                       (eq op4 '-))
                  (dformat "SOLID here9~%")
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (+ ,n1 ,n3) ,x) (- ,n2 ,n4)) parent))
                  (set-hint (fmt "~a unknowns and ~a unknowns are added.
~a is subtracted from ~a.
So add ~a and ~a first. Then multiply the unknown.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n1 n3
                                 n4 n2
                                 n1 n3
                                 n4 n2)
                            (fmt "~a Xs and ~a Xs are added.
~a is subtracted from ~a.
So add ~a and ~a first. Then multiply by X.
Finally, subtract ~a and ~a. and add it to the rest."
                                 n1 n3
                                 n4 n2
                                 n1 n3
                                 n4 n2)))
                 ((and (not (box-left-child-p b4))
                       (eq op1 '+)
                       (eq op2 '+)
                       (eq op4 '-))
                  (dformat "here10~%")
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (+ ,n1 ,(- n3)) ,x) (+ ,n2 ,n4)) parent))
                  (set-hint (fmt "Subtracting ~a unknowns is the same as adding ~a unknowns.
~a unknowns are added to ~a unknowns.
~a is added to ~a.
So add ~a to ~a first. Then multiply the unknown.
Finally, add ~a to ~a. and add it to the rest."
                                 n3 (- n3)
                                 n1 (- n3)
                                 n2 n4
                                 n1 (- n3)
                                 n2 n4)
                            (fmt "Subtracting ~a Xs is the same as adding ~a Xs.
~a Xs are added to ~a Xs.
~a is added to ~a.
So add ~a to ~a first. Then multiply X.
Finally, add ~a to ~a. and add it to the rest."
                                 n3 (- n3)
                                 n1 (- n3)
                                 n2 n4
                                 n1 (- n3)
                                 n2 n4)))
                 ((and (box-left-child-p b2)
                       (eq op1 '-)
                       (eq op2 '-)
                       (eq op4 '+))
                  (dformat "here11~%")
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (- ,n1 ,n3) ,x) (- ,(- n2) ,n4)) parent))
                  (set-hint (fmt "The right side is subtracted from the left side.
~a unknowns are subtracted from ~a unknowns.
~a is subtracted from ~a.
So subtract ~a from ~a first. Then multiply the unknown.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n3 n1
                                 n4 (- n2)
                                 n3 n1
                                 n4 (- n2))
                            (fmt "~a Xs are subtracted from ~a Xs.
~a is subtracted from ~a.
So subtract ~a from ~a first. Then multiply by X.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n3 n1
                                 n4 (- n2)
                                 n3 n1
                                 n4 (- n2))))
                 ((and (not (box-left-child-p b2))
                       (eq op1 '-)
                       (eq op2 '-)
                       (eq op4 '+))
                  (dformat "here12~%")
                  (multiple-value-setq (nb answers) (new-subtree `(+ (* (- ,(- n1) ,n3) ,x) (- ,n2 ,n4)) parent))
                  (set-hint (fmt "The right side is subtracted from the left side.
Subtracting ~a unknowns is the same as adding ~a unknowns.
~a unknowns are subtracted from ~a unknowns.
~a is subtracted from ~a.
So subtract ~a from ~a first. Then multiply the unknown.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n1 (- n1)
                                 n3 (- n1)
                                 n4 n2
                                 n3 (- n1)
                                 n4 n2)
                            (fmt "~a Xs are subtracted from ~a Xs.
~a is subtracted from ~a.
So subtract ~a from ~a first. Then multiply by X.
Finally, subtract ~a from ~a. and add it to the rest."
                                 n3 n1
                                 n4 (- n2)
                                 n3 n1
                                 n4 (- n2))))

                 (t (error "I missed one.")))
           ;; remove the boxes at the end otherwise box-left-child-p won't work.
           (remove-box-parent x b2)
           (remove-box-parent x b4)))
        (setf (arg-value carg) nb)
        (do-submit answers)))))

(defoperation subtract #\s subtract #\s (symbols nth ((b1))
                                ((:matches (:equals :arg1 (:matches :box :name b2)
                                                    :arg2 (:matches :box :name b3)
                                                    :permute t)
                                           :name b1)))
  ;;(declare (ignore nth))
  (with-matches (b1 b2 b3) symbols
    (remove-box-parent b2 b1)
    (remove-box-parent b3 b1)
    (multiple-value-bind (nb answers) (new-subtree `(- ,b2 ,b3) b1)
      (setf (arg-value (box-arg1 b1)) nb
            (arg-value (box-arg2 b1)) 0
            (arg-answer (box-arg2 b1)) 0
            (arg-editable (box-arg2 b1)) t)
      (let ((bl (if (box-left-child-p b2 b1) "left" "right"))
            (br (if (box-left-child-p b2 b1) "right" "left")))
        (set-hint (fmt "The ~a part of the diagram is subtracted
from the ~a part of the diagram.
0 is left on the ~a side."
                       bl br bl)
                  (fmt "The ~a side of the equation is subtracted
from the ~a side of the equation.
0 is left on the ~a side."
                       bl br bl)))
      (do-submit answers))))

