(in-package :tutor)

(defun make-problems (instructions hint-types problems)
  (list 'problem
        (make-instance 'problem-set
                       :instructions instructions
                       :hint-types hint-types
                       :problems problems)))

(defun make-problem (p a points &optional hint)
  (make-instance 'problem
		 :eqn p
		 :answer a
                 :points points
		 :auto-hint hint))

(defun make-text (&rest txt)
  (cons 'text txt))

(defun glob-perms (op a b)
  "A and B are lists and OP is an operator. Return a list of all
combinations of OP the elements of A and B."
  (apply #'nconc (mapcar (lambda (d)
                           (mapcar (lambda (e)
                                     (list op e d))
                                   a))
                         b)))
  
(defun perms (base)
  "Return all equivalent permutations of a problem."
  (cond ((or (typep base 'number)
	     (typep base 'symbol))
	 (list base))
	((= (length base) 1)
	 (mapcar #'list (perms (first base))))
	((= (length base) 3)
	 (let ((a (apply #'op-answers base)))
	   (apply #'nconc (mapcar (lambda (n)
				    (let ((other (when (and (find (first n) '(- +))
                                                            (numberp (second n))
                                                            (listp (third n))
                                                            (let ((m (third n)))
                                                              (and (find (first m) '(* /))
                                                                   (symbolp (third m))
                                                                   (numberp (second m)))))
                                                   ;; Handle (- N (* M X))
                                                   (let ((m (third n)))
                                                     (glob-perms (invert-op (first n)) (list (second n)) (perms `(,(first m) ,(- (second m)) ,(third m)))))))
                                          (other2 (when (and (find (first n) '(- +))
                                                             (numberp (second n))
                                                             (symbolp (third n)))
                                                    (if (eq (first n) '-)
                                                        (glob-perms '+ (list (second n)) (perms `(* -1 ,(third n))))
                                                      (glob-perms '- (list (second n)) (perms `(* -1 ,(third n)))))))
                                           (b (perms (second n)))
                                           (c (perms (third n))))
                                      (nconc other other2 (glob-perms (first n) b c))))
				  a))))
	(t (error "unknown type ~a" (type-of base)))))
	
(defun problem (prob answer points &optional hint)
  (make-problem prob (perms answer) points hint))

(defun make-docs (title &rest paragraphs)
  "Produce a list markup representation of the title and paragraphs."
  (cons (list :title title)
        (mapcar (lambda (p)
                  (list :text p))
                paragraphs)))

(defun pick-prob (p1 p2)
  (if *simplified-ui*
      p2
    p1))

(defun gen-problems (section)
  "Return a list of problems for the given section. The first ret
val is a list of practice problems. the second is the list of
problems."
  (ecase section
    (ch1-s1
     (values
      '()
      (list (problem '(* (+ 5 7) 8) '(96) 2 :base))
      (list
       (problem '(* (+ 4 9) 5) '(65) 2)
       (problem '(+ (* 9 5) 4) '(49) 2)
       (problem '(- (* 5 2) 11) '(-1) 2)
       (problem '(* (- 11 5) 2) '(12) 2)
       (problem '(- (* 7 6) 19) '(23) 2)
       (problem '(* (+ 2 5) 8) '(56) 2)
       (problem '(* (/ 54 3) 2) '(36) 2)
       (problem '(- (- 16 (* 3 2)) 4) '(6) 3)
       (problem '(- 16 (- (* 3 2) 4)) '(14) 3)
       (pick-prob
        (problem '(* (+ 10 6) (- 11 3)) '(128) 3)
        (problem '(* (+ 10 6) (- 11 7)) '(64) 3))
       (problem '(+ (* 6 (- 11 3)) 10) '(58) 3)
       (problem '(/ 100 (- (/ 20 5) 3)) '(100) 3)
       (problem '(/ 100 (/ 20 (- 5 3))) '(10) 3)
       (problem '(- (/ (- 63 8) (+ 3 8)) 2) '(3) 4)
       (problem '(* 5 (/ (- 19 7) (+ 5 1))) '(10) 4))))
    (ch1-s2
     (values
      '()
      (list 
       (problem '(+ 5 (4)) '(9) 2 :base))
      (list
       (problem '(+ (93) 5) '(98) 2)
       (problem '(+ (8) 5) '(13) 2)
       (problem '(- (/ 24 (12)) 1) '(1) 3)
       (problem '(+ (/ 72 (9)) 5) '(13) 3)
       (problem '(* (- (37) 19) 2) '(36) 3)
       (pick-prob
        (problem '(* (- (80) 31) 4) '(196) 3)
        (problem '(* (- (80) 31) 2) '(98) 3))
       (problem '(* (- (31) 31) 4) '(0) 3)
       (problem '(* 9 (13)) '(117) 2)
       (problem '(* 7 (11)) '(77) 2)
       (problem '(/ 30 (90)) '(1/3) 2)
       (problem '(/ 40 (5)) '(8) 2)
       (problem '(/ 40 (60)) '(2/3) 2)
       (problem '(/ (+ 5 (95)) 2) '(50) 3)
       (problem '(/ (+ (5) 7) 3) '(4) 3)
       (problem '(/ (+ (0) 7) 3) '(7/3) 3))))
    (ch1-s7
     (values
      '()
      (list
       (problem '(= (+ x 3) 8) '(= x 5) 2 :base))
      (list
       (problem '(= (- x 10) 17) '(= x 27) 2)
       (problem '(= (+ x 4) 13) '(= x 9) 2)
       (problem '(= (* 5 x) 90) '(= x 18) 2)
       (problem '(= (/ x 3) 29) '(= x 87) 2)
       (problem '(= (- x 16) 7) '(= x 23) 2)
       (problem '(= (- x 32) 95) '(= x 127) 2)
       (pick-prob
        (problem '(= (+ x 58) 74) '(= x 16) 2)
        (problem '(= (+ x 54) 74) '(= x 20) 2))
       (pick-prob
        (problem '(= (+ x 91) 27) '(= x -64) 2)
        (problem '(= (+ x 91) 87) '(= x -4) 2))
       (pick-prob
        (problem '(= (* 12 x) 132) '(= x 11) 2)
        (problem '(= (* 12 x) 48) '(= x 4) 2))
       (pick-prob
        (problem '(= (/ x 7) 68) '(= x 476) 2)
        (problem '(= (/ x 7) 5) '(= x 35) 2))
       (problem '(= (/ x 10) 100) '(= x 1000) 2)
       (problem '(= (+ x 5) 4) '(= x -1) 2)
       (problem '(= (- x 94) 362) '(= x 456) 2)
       (problem '(= (+ x 1984) 2001) '(= x 17) 2)
       (problem '(= (/ x 3) 51) '(= x 153) 2)
       (problem '(= (* 3 x) 51) '(= x 17) 2)
       (problem '(= (* 17 x) 51) '(= x 3) 2)
       (pick-prob
        (problem '(= (/ x 17) 51) '(= x 867) 2)
        (problem '(= (/ x 5) 50) '(= x 250) 2))
       (problem '(= (* 6 x) 2) '(= x 1/3) 2)
       (problem '(= (* 2/3 x) 6) '(= x 9) 2)
       (problem '(= (* 3/5 x) 30) '(= x 50) 2)
       (pick-prob
        (problem '(= (* 7/2 x) 28) '(= x 8) 2)
        (problem '(= (* 7 x) 28) '(= x 4) 2))
       (problem '(= (+ x 2/3) 5/3) '(= x 1) 2)
       (problem '(= (- x 3/4) 7/4) '(= x 5/2) 2))))
    (ch2-s1
     (values
      '(:verbose-minus)
      (list (list
       (problem '(+ 4 -3) '(1) 1 :base)
       (problem '(+ -3 -4) '(-7) 1 :base)))
      (list
       (problem '(+ 5 -2) '(3) 1)
       (problem '(+ 6 -4) '(2) 1)
       (problem '(+ 2 -5) '(-3) 1)
       (problem '(+ 13 -8) '(5) 1)
       (problem '(+ 3 -7) '(-4) 1)
       (problem '(+ -3 -2) '(-5) 1)
       (problem '(+ -13 -4) '(-17) 1)
       (problem '(+ -4 -5) '(-9) 1)
       (problem '(+ -8 -9) '(-17) 1)
       (problem '(+ 0 -3) '(-3) 1)
       (problem '(+ -3 5) '(2) 1)
       (problem '(+ -4 1) '(-3) 1)
       (problem '(+ -13 15) '(2) 1)
       (problem '(+ -27 1) '(-26) 1)
       (problem '(+ -11 35) '(24) 1))))
    (ch2-s3
     (values
      '(:number :subexpr :verbose-minus)
      (list
       (problem '(- -4 3) '(-7) 1 :base))
      (list
       (problem '(- 35 200) '(-165) 1)
       (problem '(- 14 -17) '(31) 1)
       (problem '(- -11 -2) '(-9) 1)
       (problem '(- -99 59) '(-158) 1)
       (problem '(- -241 -42) '(-199) 1)
       (problem '(- 158 -115) '(273) 1)
       (problem '(- 5/7 -2/7) '(1) 1)
       (problem '(- 0 -15) '(15) 1)
       (problem '(- 154 (- 43 28)) '(139) 2)
       (problem '(- 314 (- 58 70)) '(326) 2)
       (problem '(- (- 33 45) (- 64 78)) '(2) 3)
       (problem '(- (- 13 5) (- -2 6)) '(16) 3)
       (problem '(+ (- 1066 1492) 1984) '(1558) 2)
       (problem '(- (- 8 -3) (- 4 -6)) '(1) 3)
       (problem '(+ (+ (- 1 2) (- 4 8)) 16) '(11) 4)
       (problem '(- (+ (- (- 45 7) 58) 11) 21) '(-30) 4)
       (problem '(- (+ (- (+ (- 19 2) 23) 54) 34) 29) '(-9) 5)
       (problem '(- (+ (- (+ (- 200 195) 71) 66) 18) 29) '(-1) 5))))
    (ch2-s4
     (values
      '(:verbose-minus)
      (list (list
       (problem '(* -3 -7) '(21) 1 :base)
       (problem '(* -5 8) '(-40) 1 :base)))
      (list
       (problem '(* 10 -7) '(-70) 1)
       (problem '(* -5 -9) '(45) 1)
       (problem '(* -6 8) '(-48) 1)
       (problem '(* -2/3 6) '(-4) 1)
       (problem '(* -72 1) '(-72) 1)
       (problem '(* -1 -200) '(200) 1)
       (problem '(* -53 0) '(0) 1)
       (problem '(* (* -3 -5) -7) '(-105) 2)
       (problem '(* (* -2 5) -8) '(80) 2)
       (problem '(* (* 4 -7) 2) '(-56) 2)
       (problem '(* (* (* -1 5) -3) -2) '(-30) 3))))
    (ch2-s5
     (values
      '(:verbose-minus)
      (list (list 
       (problem '(/ -54 6) '(-9) 1 :base)
       (problem '(/ -54 -3) '(18) 1 :base)))
      (list
       (problem '(/ -21 7) '(-3) 1)
       (problem '(/ 21 -7) '(-3) 1)
       (problem '(/ -21 -7) '(3) 1)
       ;; 7 uses notation we don't have
       ;; 9 uses notation we don't have
       ;; 11 uses notation we don't have
       (problem '(/ -26 2) '(-13) 1)
       (problem '(/ 51 -3) '(-17) 1)
       (problem '(/ -55 -11) '(5) 1)
       (problem '(/ 1000 -10) '(-100) 1)
       (problem '(/ (- 7 19) -3) '(4) 2)
       (problem '(/ (- 13 37) -2) '(12) 2)
       (problem '(/ 45 (- 5 14)) '(-5) 2)
       (problem '(/ (- 32 4) (- 8 15)) '(-4) 3)
       (problem '(/ (* 8 -3) -6) '(4) 2)
       (problem '(/ (* -3 10) 5) '(-6) 2)
       (problem '(/ 10 -1) '(-10) 1)
       (problem '(/ -10 -1) '(10) 1)
       (problem '(/ 0 10) '(0) 1)
       (problem '(/ 0 -10) '(0) 1)
       ;; 39 is division by zero (DbZ)
       (problem '(/ 15 (5)) '(3) 2)
       (problem '(/ 15 (-3)) '(-5) 2)
       ;; 41.c is DbZ
       (problem '(/ (5) -30) '(-1/6) 2)
       (problem '(/ (-3) -30) '(1/10) 2)
       ;; 43.c is DbZ
       (problem '(/ (+ (-2) 5) (+ (-2) 3)) '(3) 3)
       (problem '(/ (+ (5) 5) (+ (5) 3)) '(5/4) 3)
       ;; 47.c is DbZ
       (problem '(/ (- 5 (2)) (- 3 (2))) '(3) 3)
       (problem '(/ (- 5 (5)) (- 3 (5))) '(0) 3)
       (problem '(/ (- 5 (-3)) (- 3 (-3))) '(4/3) 3)
       )))
    (ch2-s6
     (values
      '(:subexpr)
      (list
       (problem '(+ 3 (+ (* 2 x) 7)) '((+ (* 2 x) 10)) 2 :base))
      (list
       (problem '(+ (+ 4 x) 3) '((+ x 7)) 2)
       (problem '(+ (+ 6 x) 7) '((+ x 13)) 2)
       (problem '(- (+ 5 x) 3) '((+ x 2)) 2)
       (problem '(+ (+ -8 x) 4) '((+ x -4)) 2)
       (problem '(+ (- 15 x) 9) '((- 24 x)) 2)
       (problem '(- (- 6 x) 37) '((- -31 x)) 2)
       ;; not sure what to change or if i should change this one
       (pick-prob
        (problem '(- (- -19 x) 22) '((- -41 x)) 2)
        (problem '(- (- -19 x) 22) '((- -41 x)) 2))
       (problem '(+ (+ 9 (* 7 x)) 3) '((+ 12 (* 7 x))) 2)
       (problem '(+ (- -18 (* 3 x)) 7) '((- -11 (* 3 x) )) 2)
       (problem '(* (* 5 x) 7) '((* 35 x)) 2)
       (problem '(* (* 3 x) -8) '((* -24 x)) 2)
       (problem '(* (* 20 x) 1/5) '((* 4 x)) 2)
       (problem '(* -1/8 (* 56 x)) '((* -7 x)) 2)
       (problem '(* (* 9 x) 1/9) '((* 1 x)) 2)
       (problem '(/ (* 54 x) 9) '((* 6 x)) 2)
       (problem '(/ (* -88 x) 11) '((* -8 x)) 2)
       (problem '(/ (* 1000 x) -10) '((* -100 x)) 2)
       (problem '(/ (* -37 x) -1) '((* 37 x)) 2)
       (problem '(/ (* -19 x) -19) '((* 1 x)) 2))))
    (ch2-s7
     (values
      '()
      (list
       (problem '(= (+ (* 2 x) 3) 19) '(= x 8) 4 :base)
       (problem '(= (- 10 x) 2) '(= x 8) 2 :base))
      (list
       (problem '(= (+ (* 1/3 x) 5) 7) '(= x 6) 4)
       (problem '(= (- (* 1/2 x) 8) 3) '(= x 22) 4)
       (problem '(= (+ (* 5 x) 4) 39) '(= x 7) 4)
       (problem '(= (- (* 3 x) 7) 26) '(= x 11) 4)
       (problem '(= (+ (* -1/6 x) 5) 8) '(= x -18) 4)
       (pick-prob
        (problem '(= (- (* -1/4 x) 9) 17) '(= x -104) 4)
        (problem '(= (- (* -1/4 x) 9) 16) '(= x -100) 4))
       (problem '(= (- 13 (* 4 x)) 25) '(= x -3) 4)
       (pick-prob
        (problem '(= (- 8 (* 1/7 x)) -19) '(= x 189) 4)
        (problem '(= (- 8 (* 1/7 x)) -9) '(= x 63) 4))
       (problem '(= 53 (+ (* 5 x) 11)) '(= x 42/5) 4)
       (problem '(= (- 17 x) 25) '(= x -8) 2)
       (problem '(= 71 (- 4 x)) '(= x -67) 2)
       (problem '(= (+ (* 2/3 x) 6) 18) '(= x 18) 4)
       (problem '(= (+ -21 (* 4 x)) 10) '(= x 31/4) 4)
       (problem '(= (- -4/3 x) -1/3) '(= x -1) 2))))
    (ch3-s1
     (values
      '()
      (list
       (problem '(+ (* 5 (+ x 2)) 9) '((+ 19 (* 5 x))) 4 :base))
      (list
       (problem '(+ (* 3 (+ x 5)) 11) '((+ 26 (* 3 x))) 4)
       (problem '(+ (* 7 (+ 6 x)) 8) '((+ (* 7 x) 50)) 4)
       (problem '(+ (* 2 (+ (* 3 x) 8)) 40) '((+ (* 6 x) 56)) 6)
       (problem '(- (* 9 (+ 4 (* 8 x))) 6) '((+ (* 72 x) 30)) 6)
       (problem '(+ 100 (* 6 (+ (* 4 x) 9))) '((+ (* 24 x) 154)) 6))))
    (ch3-s2 
     (values
      '(:subexpr)
      (list
       (problem '(- 9 (* 2 (- x 4))) '((+ (* -2 x) 17)) 4 :all))
      (list
       ;; 3-2
       (problem '(* 3 (+ x 7)) '((+ (* 3 x) 21)) 2)
       (problem '(* 7 (- (* 3 x) 8)) '((- (* 21 x) 56)) 4)
       ;; missing 5
       (problem '(* -5 (- 6 x)) '((- -30 (* -5 x))) 2)
       ;; missing 9
       ;;      (problem '(* x (+ x 9)) '((+ (* x x) (* x 9))))
       (problem '(* 6 (+ 9 (* 7 x))) '((+ 54 (* 42 x))) 4)
       ;; missing 15
       (problem '(* 28 (+ (* 1/4 x) 1/7)) '((+ (* 7 x) 4)) 4)
       (problem '(* 2/3 (+ (* 4 x) 3/4)) '((+ (* 8/3 x) 1/2)) 4)
       (problem '(- 8 (* 5 (- 4 (* 6 x)))) '((+ -12 (* 30 x))) 6)
       (problem '(- 11 (* 1 (+ (* 5 x) 3))) '((+ 8 (* -5 x))) 6))))
    (ch3-s4
     (values
      '()
      (list
       (problem '(- (* 7 x) (* 5 x)) '((* 2 x)) 2 :base)
       ;; This is perhaps Not Good. I made this example up.
       (problem '(+ (* 5 x) (- 6 (* 2 x))) '((+ (* 3 x) 6)) 2 :all))
      (list
       (problem '(+ (* 5 x) (* 9 x)) '((* 14 x)) 2)
       (problem '(- (* 8 x) (* 3 x)) '((* 5 x)) 2)
       (problem '(- (* 9 x) (* 8 x)) '((* 1 x)) 2)
       (problem '(- (* 6 x) x) '((* 5 x)) 2)
       (problem '(+ (+ (* 3 x) (* 7 x)) 8) '((+ (* 10 x) 8)) 2)
       (problem '(+ (+ (* 5 x) 9) (* 7 x)) '((+ (* 12 x) 9)) 2)
       (problem '(- (- (* 3 x) 2) (* 8 x)) '((- (* -5 x) 2)) 2)
       (problem '(+ (- -4 (* 5 x)) (* 11 x)) '((+ -4 (* 6 x))) 2)
       (problem '(- (+ (* -6 x) 13) (* 7 x)) '((+ 13 (* -13 x))) 2)
       ;; This one might be too hairy
       ;;(problem '(+ (- (* 45/10 x) (* 71/10 x)) 116/10) '((+ 116/10 (* 116/10 x))) 1)
       ;; skip to #41
       ;; this uses reorder
       (problem '(+ (* 5 x) (* 3 (- x 2))) '((- (* 8 x) 6)) 4)
       (problem '(- 7 (* 4 (+ (* 2 x) 5))) '((- -13 (* 8 x))) 6)
       (problem '(- (* 6 x) (- x 1)) '((+ (* 5 x) 1)) 2)
       (problem '(- (* 2 (+ (* 3 x) 4)) (* 5 x)) '((+ (* 1 x) 8)) 6)
       (problem '(+ (* -8 (- (* 9 x) 7)) (- (* 72 x) 57)) '(-1) 9)
       (problem '(+ (* 4 (+ (* 3 x) 2)) (* 6 (+ (* 5 x) 8))) '((+ (* 42 x) 56)) 11)
       (problem '(+ (* 2 (- x 5)) (* 5 (- 4 (* 2 x)))) '((+ (* -8 x) 10)) 9)
       (problem '(- (* 8 (+ (* 2 x) 3)) (* 4 (+ (* 3 x) 6))) '((+ (* 4 x) 0)) 11)
       (problem '(- (* 3 (+ 6 (* 5 x))) (* 7 (- (* 2 x) 1))) '((+ (* 1 x) 25)) 11)
       (problem '(- (* -6 (- x 5)) (* 2 (- 7 (* 3 x)))) '(16) 11)
       (problem '(- (* 8 (+ x 4)) (- x 5)) '((+ (* 7 x) 37)) 5)
       ;; skip 63
       ;; this prolly won't work
       ;;(problem '(+ (* 3 x) (* (x (- 5 (* 2 x)))) '() 1)
       (problem '(- (* 1/4 (+ (* 12 x) 20)) (* 1/5 (- 30 (* 15 x)))) '((+ (* 6 x) -1)) 11)
       ;; skip 69
       )))
    (ch4-s1 
     (values 
      '()
      (list
       (problem '(= (+ (* 2 x) (* 6 x)) 56) '(= x 7) 4 :base))
      (list
       (problem '(= (- (* 7 x) (* 4 x)) 15) '(= x 5) 4)
       (problem '(= (+ (+ (* 3 x) (* 4 x)) 8) 22) '(= x 2) 6)
       (problem '(= (+ (- (* 10 x) (* 7 x)) 18) 6) '(= x -4) 6)
       (problem '(= (+ (- (* 5 x) (* 7 x)) 21) 27) '(= x -3) 6)
       (problem '(= (+ (+ (* 7 x) 3) (* 3 x)) 63) '(= x 6) 6)
       (problem '(= (- (- (* 9 x) 14) (* 5 x)) -10) '(= x 1) 6)
       (problem '(= (- (* 10 x) x) 90) '(= x 10) 4)
       (problem '(= (- (- (* 7 x) 4) (* 6 x)) 5) '(= x 9) 6)
       (problem '(= (+ (+ (+ (* 2 x) (* 3 x)) 4) (* 5 x)) 34) '(= x 3) 8)
       (problem '(= (+ (+ (+ (* 3 x) 4) (* 5 x)) (* 6 x)) 32) '(= x 2) 8))))
    (ch4-s2
     (values
      '()
      (list 
       (problem '(= (+ (* 6 (- x 2)) (* 5 x)) 43) '(= x 5) 8 :base))
      (list
       (problem '(= (+ (* 7 x) (* 3 x)) 40) '(= x 4) 4)
       (pick-prob
        (problem '(= (- (* 4 x) (* 11 x)) 105) '(= x -15) 4)
        (problem '(= (- (* 4 x) (* 11 x)) 49) '(= x -7) 4))
       (problem '(= (+ (+ (* 16 x) 3) (* 4 x)) 103) '(= x 5) 6)
       (problem '(= (- (- (* 7 x) 8) (* 4 x)) 25) '(= x 11) 6)
       (problem '(= (- (+ (* -5 x) 17) (* 8 x)) 56) '(= x -39/13) 6)
       (problem '(= 58 (+ (- 6 (* 14 x)) (* 12 x))) '(= x -26) 6)
       (problem '(= (- (+ (* 8 x) 5) (* 7 x)) 22) '(= x 17) 6)
       (problem '(= (- (+ (* 20 x) 84) (* 8 x)) 0) '(= x -7) 6)
       (problem '(= (+ (+ x 21) (* 4 x)) -33) '(= x -54/5) 6)
       (problem '(= (+ (+ (- (* 2 x) (* 5 x)) 13) (* 9 x)) 67) '(= x 9) 8)
       (problem '(= (- (* 5 (+ x 3)) (* 2 x)) -21) '(= x -12) 8)
       (problem '(= (+ (* 2 (- (* 3 x) 7)) (* 4 x)) 26) '(= x 4) 10)
       (problem '(= (+ (* 5 x) (* 3 (+ x 4))) 28) '(= x 2) 8)
       (problem '(= (- (* 7 x) (* 4 (- 2 (* 3 x)))) -27) '(= x -1) 10)
       (problem '(= (- (* 7 x) (+ 5 (* 6 x))) 4) '(= x 9) 6)
       (problem '(= (+ (- 12 (* 7 (- x 4))) x) -2) '(= x 7) 10)
       (problem '(= (- (* 2 (+ x 3)) (* 5 (- x 1))) 32) '(= x -7) 11)
       (problem '(= 55 (+ (* 3 (- (* 2 x) 1)) (* 2 (+ x 5)))) '(= x 6) 13)
       (problem '(= 0 (+ (* 4 (- 6 x)) (* 7 x))) '(= x -8) 8)
       (problem '(= (- (* 3 (+ x 2)) (- x 1)) 17) '(= x 5) 9))))
    (ch4-s3
     (values
      '()
      (list
       (problem '(= (+ (* 3 x) 55) (* 8 x)) '(= x 11) 7 :base))
      (list
       (problem '(= (+ (* 5 x) 27) (* 2 x)) '(= x -9) 7)
       (problem '(= (- (* 6 x) 28) (* 8 x)) '(= x -14) 7)
       (problem '(= (* 9 x) (- (* 4 x) 65)) '(= x -13) 7)
       (problem '(= (- 55 (* 3 x)) (* 8 x)) '(= x 5) 7)
       (problem '(= (* 5 x) (- 48 x)) '(= x 8) 7)
       (problem '(= (- (* 10 x) 51) (* 7 x)) '(= x 17) 7)
       (problem '(= (+ (* 29 x) 56) (* 27 x)) '(= x -28) 7)
       (problem '(= (* -6 x) (- x 70)) '(= x 10) 7)
       (problem '(= (* 9 x) (+ 34 (* 8 x))) '(= x 34) 7)
       (problem '(= (* 7 x) (- -16 (* 9 x))) '(= x -1) 7)
       (problem '(= (+ (* 5 x) 8) (+ (* 7 x) 8)) '(= x 0) 8)
       (problem '(= (- (* 2 x) 5) (+ (* 3 x) 4)) '(= x -9) 8)
       (problem '(= (- (+ (* 6 x) 7) (* 2 x)) (- (+ 3 (* 2 x)) 9)) '(= x -13/2) 12)
       (problem '(= (* 4 (+ x 3)) (* 6 x)) '(= x 6) 9)
       (pick-prob
        (problem '(= (* 5 (- 9 x)) (* 4 (+ x 18))) '(= x -3) 12)
        (problem '(= (* 3 (- 9 x)) (* 2 (+ x -4))) '(= x 7) 12))
       (problem '(= (+ (* 3 x) (* 3 (- 1 x))) (- x 17)) '(= x 20) 8)
       (problem '(= (* 2 (- 1 (* 3 (+ x 2)))) (- x 1)) '(= x -9/7) 18)
)))))

(defun gen-follow-instructions (simplify-p)
  (if (or (tutor2-p)
          *simplified-ui*)
      (format nil "Follow the instructions below the problem to ~a."
              (if simplify-p
              "simplify it as much as possible"
              "solve it"))))

(defun gen-instructions-tutor2 (section)
  (case section
    (ch1-s1
     (list
      (make-docs "Evaluate an Expression"
                 "To evaluate an expression find an operation with all numbers,
evaluate the operation, and repeat until there are no more
operations."
                 (gen-follow-instructions nil))))
    (ch1-s2
     (list
      (make-docs "Variables"
                 "A variable is a letter that represents a number. To
evaluate an expression first substitute the variable with its
number and then go on."
                 (gen-follow-instructions nil))))
    (ch1-s7
     (list
      (make-docs "Solving Equations"
                 "An equation is a sentence which says an expression with X is
equal to a number. To solve an equation, find the value
of X that would make this true. To do this the equation must be 
transformed so that X is alone on one side of the equal sign and a 
number is on the other side. You can do this by unwinding the operations on the side of the 
equation that contains X."
                 (gen-follow-instructions nil))))
    (ch2-s1
     (list
      (make-docs "Adding Negative Numbers"
                 "The absolute value of a number is the positive version of the number."
                 "If the two numbers have opposite signs, subtract the absolute values. The answer has the sign of the term with the greater absolute value."
                 "If the two numbers have the same sign, add the absolute values. Use the sign of the two numbers for the answer. Positive plus positive is positive. Negative plus negative is negative."
                 (gen-follow-instructions nil))))
    (ch2-s3
     (list
      (make-docs "Subtracting Signed Numbers"
                 "Change subtracting to adding the opposite. Then follow the rules for adding signed numbers."
                 (gen-follow-instructions nil))))
    (ch2-s4
     (list
      (make-docs "Multiplying Signed Numbers"
                 "Negative times negative is positive."
                 "Positive times negative is negative."
                 "Negative times positive is negative."
                 (gen-follow-instructions nil))))
    (ch2-s5
     (list
      (make-docs "Division Of Signed Numbers"
                 "Negative divided by negative is positive."
                 "Positive divided by negative is negative."
                 "Negative divided by positive is negative."
                 (gen-follow-instructions nil))))
    (ch2-s6
     (list
      (make-docs "Collecting Like Terms"
                 "When one of the expressions in an addition or subtraction is itself 
an addition or subtraction, you can combine the two operations. You 
can likewise combine two multiplications."
                 (gen-follow-instructions t))))
    (ch2-s7
     (list
      (make-docs "Equations That Need Two Transformations"
                 "The unwind transformation can be repeated."
                 "Follow the instructions below the problem to solve it.")
      (make-docs "Invert Special Case"
                 "There is a special case when the operator is - and the variable is on the right."
                 (gen-follow-instructions nil))))
    (ch3-s1
     (list
      (make-docs "The Distribute Transformation"
                 "If an expression has parenthesis in it, the operation inside the parenthesis should be done first."
                 "But, if the operation inside is addition and the operation outside is multiplication, you can multiply each number inside the parenthesis by the number outside the parenthesis and then add the answers."
                 (gen-follow-instructions t))))
    (ch3-s2
     (list
      (make-docs "Distributing over subtraction"
                 "You can also distribute when the operation inside the parenthesis is subtraction instead of addition."
                 (gen-follow-instructions t))))
    (ch3-s4
     (list
      (make-docs "Like Terms"
                 "The numeric coefficient of a term is the number that is multiplied by the variable. When two variables with numeric coefficients are added or subtracted you can use the Collect transformation to combine the numeric coefficients."
                 "Follow the instructions above the problem to simplify it as much as possible.")
      (make-docs "Like Terms"
                 "Collect works as long as all the operations between the variables are subtraction or addition."
                 (gen-follow-instructions t))))
    (ch4-s1
     (list
      (make-docs "Equations With Like Terms"
                 ;;"The Collect transformation can be used not only to collect numbers. It can be used to collect variables."
                 (gen-follow-instructions nil))
;;       (make-docs "Equations With Like Terms"
;;                  "Collect works when there is an extra + or - operation between the variables, too."
;;                  "Follow the instructions above the problem to simplify it as much as possible.")
      ))
    (ch4-s2
     (list
      (make-docs "Harder Equations"
                 (gen-follow-instructions nil))))
    (ch4-s3
     (list
      (make-docs "The Subtract Transformation"
                 "Sometimes the variable appears on both sides of the equation. To transform this into an equation one can solve, subtract the right side from the left and say that this difference is 0."
                 (gen-follow-instructions nil))))))

(defun gen-instructions-tutor1 (section)
  (case section
    (ch1-s1
     (list
      (make-docs "Evaluate a Diagram"
                 "To evaluate a diagram find an operation with all numbers, evaluate the operation, copy the results according to the arrow, and repeat until it becomes a number."
                 (gen-instructions-tutor1 nil))))
    (ch1-s2
     (list
      (make-docs "Input Boxes"
                 "Input boxes simply contain numbers. To evaluate a diagram first copy the number in the input box according to the input arrows and then go on."
                 (gen-instructions-tutor1 nil))))
    (ch1-s7
     (list
      (make-docs "Finding Input Values"
                 "Sometimes a diagram contains an input box without a value
in it and you must find that value. If there is a number in the
box at the bottom, a number in the box feeding into it, and the
unknown box feeds into the second box you can invert the
boxes to find the value."
                 (gen-instructions-tutor1 nil))))
    ((ch2-s1 ch2-s3 ch2-s4 ch2-s5)
     (list
      (make-docs "no docs")))
    (ch2-s6
     (list
      (make-docs "The Combine Operator"
                 "If an operator box with one number feeds into another operator box with one number and their operators are either + or -, or either * or / then you can use Combine to eliminate one operator."
                 (gen-instructions-tutor1 t))))
    (ch2-s7
     (list
      (make-docs "More On Invert"
                 "The invert operation can be repeated."
                 (gen-instructions-tutor1 nil))
      (make-docs "Invert Special Case"
                 "There is a special case when the operator is - and the unknown is on the right."
                 (gen-instructions-tutor1 nil))))
    (ch3-s1
     (list
      (make-docs "The Reorder Operator"
                 "If a + or - operator box feeds into a * operator box whose other operand is a number, the Reorder operator can be used to reverse the order of the operators."
                 (gen-instructions-tutor1 t))))
    (ch3-s2
     (list
      (make-docs "Reorder And Subtraction."
                 "You can also reorder if - operator box feeds into a *  operator box."
                 (gen-instructions-tutor1 t))))
    (ch3-s4
     (list
      (make-docs "Unknown Boxes"
                 "Sometimes an unknown box will have more than one arrow
connecting it to other boxes. If the boxes connected to
it are * operator boxes, and they are connected to a
+ or - operator box, then you can use Combine to
simplify the diagram."
                 (gen-instructions-tutor1 t))
      (make-docs "Unknown Boxes"
                 "Combine works when there is an extra + or - operator box, too."
                 (gen-instructions-tutor1 t))))
    (ch4-s1
     (list
      (make-docs "Find the Value of the Unknown"
                 (gen-instructions-tutor1 nil))))
    (ch4-s2
     (list
      (make-docs "Harder Diagrams"
                 (gen-instructions-tutor1 nil))))
    (ch4-s3
     (list
      (make-docs "The Subtract Operation"
                 "When two boxes feed into the same result box at the bottom this
means they are equal.  One can transform this into a diagram we
can operate on by subtracting the right box from the left and
saying that this difference is 0"
                 (gen-instructions-tutor1 nil))))))

(defun get-section (section &optional tutor)
  (let ((solve (make-docs "Solve the following problems"
                          "If you get stuck, click the Hint button for a hint."
                          "If you make a mistake but can't find it, click the First Mistake button to jump back to it."))
        (simplify (make-docs (format nil "Simplify the following problems~%as much as possible")
                             "If you get stuck, click the Hint button for a hint."
                             "If you make a mistake but can't find it, click the First Mistake button to jump back to it.")))
    (let ((instr (if tutor
                     (gen-instructions-tutor1 section)
                   (gen-instructions-tutor2 section))))
      (multiple-value-bind (hint-types p1 p2) (gen-problems section)
        (append
         (mapcar (lambda (i p)
                   (make-problems i hint-types (if (listp p)
                                                   p
                                                 (list p)))) instr p1)
         (list
          (make-problems
           ;; do they solve or simplify them?
           (if (find section '(ch2-s6 ch3-s1 ch3-s2 ch3-s4))
               simplify
             solve)
           hint-types
           p2)))))))

;; For backwards compatibility, generate the chN-sM functions
(macrolet ((defsection (section)
             `(defun ,section ()
                (get-section ',section (tutor1-p)))))
  (defsection ch1-s1)
  (defsection ch1-s2)
  (defsection ch1-s7)
  (defsection ch2-s1)
  (defsection ch2-s3)
  (defsection ch2-s4)
  (defsection ch2-s5)
  (defsection ch2-s6)
  (defsection ch2-s7)
  (defsection ch3-s1)
  (defsection ch3-s2)
  (defsection ch3-s4)
  (defsection ch4-s1)
  (defsection ch4-s2)
  (defsection ch4-s3))

(defun do-section (ch section &optional (subject 1))
  "Run the tutor on chapter CH, section SECTION."
  (check-type ch number)
  (check-type section number)
  (let ((sym (intern (format nil "CH~a-S~a" ch section) "TUTOR")))
    (tutor subject (nconc
                    (funcall sym)
                    '((end))))))

(defun do-problem (eqn &optional (answer '(1)) (subject 1))
  (tutor subject
         (list
          (make-problems 
           '((:title "Problem")
             (:text "Solve the problem."))
           '()
           (list
            (problem eqn answer 1))))))


;; (defmacro defsection (section solvep &rest instruction)
;;   (with-gensyms (p1 p2)
;;     `(defun ,section ()
;;        (multiple-value-bind (,p1 ,p2) (gen-problems ',section)
;;          (list
;;           ,@(loop for i from 0 below (length instruction)
;;                       collect `(make-problems ,(nth i instruction)
;;                                               (list (nth ,i ,p1))))
;;           (make-problems
;;            ',(if solvep
;;                  '(make-docs "Solve the following problems."
;;                              "If you get stuck, click the Hint button for a hint."
;;                              "If you make a mistake but can't find it, click the First Mistake
;; button to jump back to it.")
;;                '(make-docs "Simplify the following problems as much as possible."
;;                            "If you get stuck, click the Hint button for a hint."
;;                            "If you make a mistake but can't find it, click the First Mistake
;; button to jump back to it."))
;;            ,p2))))))

;; (defsection ch1-s1 t 
;;   (make-docs "Evaluate a Diagram"
;;              "To evaluate a diagram find an operation with all numbers,
;; evaluate the operation, copy the results according to the
;; arrow, and repeat until it becomes a number."
;;              "Follow the instructions above the problem to solve it."))

;; (defsection ch1-s2 t
;;   (make-docs "Input Boxes"
;;              "Input boxes simply contain numbers. To evaluate a diagram first
;; copy the number in the input box according to the input arrows
;; and then go on."
;;              "Follow the instructions above the problem to solve it."))

;; (defsection ch1-s7 t
;;   (make-docs "Finding Input Values"
             
;;      "Sometimes a diagram contains an input box without a value
;; in it and you must find that value. If there is a number in the
;; box at the bottom, a number in the box feeding into it, and the
;; unknown box feeds into the second box you can invert the
;; boxes to find the value."
;;      "Follow the instructions above the problem to solve it."))

;; (defsection ch2-s6 nil
;;   (make-docs "The Combine Operator"
;;              "If an operator box with one number feeds into another operator
;; box with one number and their operators are either + or -, or
;; either * or / then you can use Combine to eliminate one
;; operator."
;;              "Follow the instructions above the problem to simplify it as much as possible."))

;; (defsection ch2-s7 t
;;   (make-docs "More On Invert"
;;              "The invert operation can be repeated."
;;              "Follow the instructions above the problem to solve it.")
;;   (make-docs "Invert Special Case"
;;              "There is a special case when the operator is - and the unknown is
;; on the right."
;;              "Follow the instructions above the problem to solve it."))

;; (defsection ch3-s1 nil
;;   (make-docs "The Reorder Operator"
;;              "If a + or - operator box feeds into a * operator box whose other
;; operand is a number, the Reorder operator can be used to
;; reverse the order of the operators."
;;              "Follow the instructions above the problem to simplify it as much as possible."))

;; (defsection ch3-s2 nil
;;   (make-docs "The Minus Operation."
;;              "Some diagrams contain minus operators which make
;; the Combine operation difficult to perform. Use the Minus
;; operation to change subtraction to adding a negative."))

;; (defsection ch3-s4 nil
;;   (make-docs "Unknown Boxes"
;;              "Sometimes an unknown box will have more than one arrow
;; connecting it to other boxes. If the boxes connected to
;; it are * operator boxes, and they are connected to a
;; + or - operator box, then you can use Combine to
;; simplify the diagram."
;;              "Follow the instructions above the problem to simplify it as much as possible.")
;;   (make-docs "Unknown Boxes"
;;              "Combine works when there is an extra + or - operator
;; box, too."
;;              "Follow the instructions above the problem to simplify it as much as possible."))

;; (defsection ch4-s1 t
;;   (make-docs "Find the Value of the Unknown"
;;              "Follow the instructions above the problem to simplify it as much as possible."))
  
;; (defsection ch4-s2 t
;;   (make-docs "Harder Diagrams"
;;              "Follow the instructions above the problem to solve it."))

;; (defsection ch4-s3 t
;;   (make-docs "The Subtract Operation"
;;              "When two boxes feed into the same result box at the bottom this
;; means they are equal.  One can transform this into a diagram we
;; can operate on by subtracting the right box from the left and
;; saying that this difference is 0"
;;              "Follow the instructions above the problem to solve it."))


(defun build-session-sections (sections)
  (apply #'nconc (loop for s on (mapcar #'funcall sections)
                       collect (list (car s)
                                     (if (cdr s)
                                         '((break))
                                       '((end)))))))
(defun tutor1-session-data (n)
  (ecase n
    (1 (values '(ch1-s1 ch1-s2 ch1-s7 ch2-s6 ch2-s7)
               "14.98"))
    (2 (values '(ch3-s1 ch3-s2 ch3-s4 ch4-s1)
               "15.82"))
    (3 (values '(ch4-s2 ch4-s3)
               "22.54"))
    (10 (values '(ch1-s1 ch1-s2 ch1-s7 ch2-s1 ch2-s3 ch2-s4 ch2-s5 ch2-s6 ch2-s7 ch3-s1 ch3-s2 ch3-s4 ch4-s1 ch4-s2 ch4-s3)
                "0.00"))))

(defun tutor2-session-data (n)
   (ecase n
     (1 (values '(ch1-s1 ch1-s2 ch1-s7 ch2-s1 ch2-s3 ch2-s4 ch2-s5 ch2-s6 ch2-s7)
                "23.31"))
     (2 (values '(ch3-s1 ch3-s2 ch3-s4 ch4-s1)
                "15.82"))
     (3 (values '(ch4-s2 ch4-s3)
                "22.54"))
     (10 (values '(ch1-s1 ch1-s2 ch1-s7 ch2-s1 ch2-s3 ch2-s4 ch2-s5 ch2-s6 ch2-s7 ch3-s1 ch3-s2 ch3-s4 ch4-s1 ch4-s2 ch4-s3)
                 "0.00"))))

(defun tutor1-session (n id)
  (multiple-value-bind (sections money) (tutor1-session-data n)
    (tutor id
           (append
            `((text ((:title ,(format nil "Algebra Tutor Session ~a" n))
                     (:text "In this experiment you will solve a number of problems given some instruction.")
                     (:text "In each problem there will be boxes containing arithmetic
expressions like 3+4. Some boxes will be connected to other boxes
with arrows. To solve the problem you will use different
operations to transform the diagram into the answer.")
                     (:text ,(format nil "Session ~a problems are broken up into ~a sections. The
first and sometimes the second problem in each section is a practice problem. The tutor
will guide you through each step. After the practice problem you
will be given a number of real problems to solve." n (length sections)))
                     (:text "Instructions will appear in the top-left corner of the tutor window. Read them carefully."))
                    ((:title "The Incentive System")
                     (:text ,(format nil "Each problem is worth a certain amount of money based on
how many operations are needed to solve it. If you complete every problem perfectly you
will make $~a. In the lower right corner of the tutor window
you can see how much money you've made so far." money))
                     (:text "Each operation is worth 7 cents. A mistake deducts some amount from the problem's total worth.")
                     (:text "If you choose an operation that can't be performed with
the boxes you've selected, you will lose 3 cents for the problem.")
                     (:text "If you select the right boxes and choose the right
operation, but fill in the wrong values you will lose 7 cents.")
                     (:text "You can never make less than $0 on a problem.")
                     (:text "Practice problems are not rewarded."))
                    ((:title "Hints and Mistakes")
                     (:text "If you get stuck or are unsure of what to do next, you
can ask the tutor for a hint by clicking the 'Hint' button. A hint costs 4 cents.")
                     (:text "The tutor will not let you advance to the next problem
until you have solved the current one correctly. If you make a
mistake you can use the 'First Mistake' button to jump back to
the diagram state after your mistake. The diagram on the left side of the tutor is the diagram state before your mistake was made. Use it
with the diagram on the right to identify your error. Once you have, click the '<-' button to go
back, then redo the operation correctly."))
                    ((:title "The Calculator")
                     (:text "You have a calculator at your disposal to help you with the arithmetic. Take a moment to familiarize yourself with its functionality. Note that the  button is used to create fractions."))
                    ((:title "Questions?")
                     (:text "If you have any questions, ask your experimenter now. If not click 'Begin' to start the tutor."))))
            (apply #'append (build-session-sections sections))))))

(defun tutor2-session (n id)
  (multiple-value-bind (sections money) (tutor2-session-data n)
    (tutor id
           (append
            `((text ((:title ,(format nil "Algebra Tutor Session ~a" n))
                     (:text "In this experiment you will solve a number of problems given some instruction.")
                     (:text "Each problem will have arithmetic expressions in it such as 3+4. To solve the problem you will use different
operations to transform the problem into the answer.")
                     (:text ,(format nil "Session ~a problems are broken up into ~a sections. The
first and sometimes the second problem in each section is a practice problem. The tutor
will guide you through each step. After the practice problem you
will be given a number of real problems to solve." n (length sections)))
                     (:text "Instructions will appear in the top-left corner of the tutor window. Read them carefully."))
                    ((:title "The Incentive System")
                     (:text ,(format nil "Each problem is worth a certain amount of money based on
how many operations are needed to solve it. If you complete every problem perfectly you
will make $~a. In the lower right corner of the tutor window
you can see how much money you've made so far." money))
                     (:text ,(format nil "Each operation is worth ~d cents. Mistakes cost you." +operation-points+))
                     (:text ,(format nil "If you choose an operation that can't be performed with
the operators you've selected, you will lose ~d cents for the problem." +bad-operation-points+))
                     (:text ,(format nil "If you select the right operators and choose the right
operation, but fill in the wrong values you will lose ~d cents." +bad-fill-in+))
                     (:text "You can never make less than $0 on a problem.")
                     (:text "Practice problems are not rewarded."))
                    ((:title "Hints and Mistakes")
                     (:text ,(format nil "If you get stuck or are unsure of what to do next, you
can ask the tutor for a hint by clicking the 'Hint' button. A hint costs ~d cents." +hint-points+))
                     (:text "The tutor will not let you advance to the next problem
until you have solved the current one correctly. If you make a
mistake you can use the 'First Mistake' button to jump back to
the problem state after your mistake. Use it
with the line above to identify your error. Once you have, click the '<-' button to go
back, then redo the operation correctly."))
                    ((:title "The Calculator")
                     (:text "You have a calculator at your disposal to help you with the arithmetic. Take a moment to familiarize yourself with its functionality. Note that the  button is used to create fractions."))
                    ((:title "Questions?")
                     (:text "If you have any questions, ask your experimenter now. If not click 'Begin' to start the tutor."))))
            (apply #'append (build-session-sections sections))))))

;; Generate the sessions
(macrolet ((defsession (n)
             (with-gensyms (id)
               `(defun ,(intern (format nil "SESSION~a" n) "TUTOR") (,id)
                  (if (tutor1-p)
                      (tutor1-session ,n ,id)
                    (tutor2-session ,n ,id))))))
  (defsession 1)
  (defsession 2)
  (defsession 3)
  ;; session 10 contains all the sections
  (defsession 10))
