;;; Click regions

(in-package :tutor)

(defstruct click-region
  view x1 y1 x2 y2 id action)

(defun register-click-region (x1 y1 x2 y2 id action)
  "Given a square and a function to call when a click event lands in
the region, add it to the list of click-regions."
  (push (make-click-region :x1 x1 :y1 y1 :x2 x2 :y2 y2 :id id :action action) (tutor-click-regions *tutor*)))

(defun register-click-view (view id action)
  (push (make-click-region :view view :id id :action action) (tutor-click-regions *tutor*)))

(defun clear-click-regions ()
  (setf (tutor-click-regions *tutor*) '()))

(defun in-reg (x y reg)
  (if (click-region-view reg)
      (let* ((view (click-region-view reg))
             (p (convert-coordinates #@(0 0) view (tutor-eqn-view *tutor*)))
             (x1 (point-h p))
             (y1 (point-v p))
             (x2 (+ (point-h p) (view-width view)))
             (y2 (+ (point-v p) (view-height view))))
        (not (or (> x x2)
                 (< x x1)
                 (> y y2)
                 (< y y1))))
    (not (or (> x (click-region-x2 reg))
             (< x (click-region-x1 reg))
             (> y (click-region-y2 reg))
             (< y (click-region-y1 reg))))))

(defun handle-click-event (x y)
  (dformat "handle-click-event: ~a ~a~%" x y)
  (dolist (reg (tutor-click-regions *tutor*))
    (when (in-reg x y reg)
      ;;(history-add-click t x y)
      (return-from handle-click-event (funcall (click-region-action reg)))))
  ;; Log the unsuccessful click
  (history-add-click nil x y))

(defun handle-click-region (view)
  (let ((reg (find view (tutor-click-regions *tutor*) :key 'click-region-view)))
    (when reg
      (funcall (click-region-action reg)))))

(defgeneric register-boxes (root)
  (:documentation "Register all the clickable boxes in root."))

(defmethod register-boxes ((box number))
  (declare (ignore box))
  nil)

(defmethod register-boxes ((box x-box))
  (declare (ignore box))
  nil)

(defmethod register-boxes ((box answer-box))
  (register-boxes (arg-value (answer box))))

(defmethod register-boxes ((box number-box))
  (register-click-region (box-x box) (box-y box) (+ (box-x box) +box-width+) (+ (box-y box) +box-height+)
                         (box-id box)
			 (lambda () (click-box box)))
  (register-boxes (arg-value (number box))))

(defmethod register-boxes ((box arg-box))
  (register-click-region (box-x box) (box-y box) (+ (box-x box) +box-width+) (+ (box-y box) +box-height+)
                         (box-id box)
			 (lambda () (click-box box)))
  (register-boxes (arg-value (box-arg1 box)))
  (register-boxes (arg-value (box-arg2 box))))
