;; pm device for tutor1

(in-package :tutor)

;; we may not need this since we can specialize on tutor-window and
;; generate all our own features. This has the advantage of not having
;; to write all the mouse clicking/moving code. The problem is that we
;; still need to deal with the calculator and input box which are
;; seperate windows.
(defclass tutor-vision-module (cl-user::vision-module)
  ())

(defmacro def-feature (name &rest slots)
  (let ((kind (intern (symbol-name name) :cl-user))
        (value (intern (symbol-name name) :cl-user)))
    `(defclass ,(intern (concat (symbol-name name) "-FEATURE") :tutor) (cl-user::icon-feature)
       ,(mapcar (lambda (slot)
                  (list slot :accessor slot :initarg (intern (symbol-name slot) :keyword)))
                slots)
       (:default-initargs
        :kind ',kind
        :value ',value
        :dmo-id (gentemp ,(symbol-name name))))))

;; Modified by AS, Feb 1st, 2006
;; Symbols moved to cl-user

(def-feature cl-user::text)
(def-feature cl-user::rectangle)
;; userprop1: from, userprop2: to
(def-feature cl-user::arrow)
(def-feature cl-user::redarrow)
(def-feature cl-user::button)
(def-feature cl-user::input-dialog)
(def-feature cl-user::calculator)
;; userprop1: left-arg, userprop2: op, userprop3: right-arg
(def-feature cl-user::arg-box)
;; userprop1: answer
(def-feature cl-user::answer-box)
;; userprop1: cl-user::number
(def-feature cl-user::number-box)
(def-feature cl-user::x-box)     ;; W PlayStation

;; tutor2 specific features
(def-feature expression)

(defun build-instruction-features ()
  (let ((v (tutor-instruction-view *tutor*)))
    (when v
      (list
       (make-instance 'text-feature
                      :value (instructions (tutor-problem-set *tutor*))
                      :x (abs-x v)
                      :y (abs-y v)
                      :width (view-width v)
                      :height (view-height v))))))

(defun build-button-features ()
  ;; just find all the buttons.
  (let ((bs (all-subviews *window* 'button-dialog-item)))
    (mapcar (lambda (b)
              (make-instance 'button-feature
                             :value (string-downcase (dialog-item-text b))
                             :x (+ (abs-x b) (/ (view-width b) 2)) 
                             :y (+ (abs-y b) (/ (view-height b) 2))
                             :width (view-width b)
                             :height (view-height b)))
            bs)))


(defun build-box-contents (box view)
  (format t "~%This is the box input: ~A" box)
  (labels ((single-arg (arg)
                       (list (make-instance 'rectangle-feature
                                            :color 'cl-user::green
                                            :value (get-proper-id box)
                                            :x (+ (abs-x view) (view-x (arg-widget arg)) (round (/ (view-width (arg-widget arg)) 2)))
                                            :y (+ (abs-y view) (view-y (arg-widget arg)) (round (/ (view-height (arg-widget arg)) 2)))
                                            :width (view-width (arg-widget arg))
                                            :height (view-height (arg-widget arg))))))
    (cond ((or (arg-op-editable-p box)
               (op-arg-editable-p box)
               (all-editable-p box))
           (list (make-instance 'rectangle-feature
                                :color 'cl-user::green
                                :x (+ (abs-x view) (view-x (box-answer-view box)))
                                :y (+ (abs-y view) (view-y (box-answer-view box)))
                                :width (view-width (box-answer-view box))
                                :height (view-height (box-answer-view box)))))
          ((op-editable-p box)
           (single-arg (box-op box)))
          ((arg-editable (box-arg1 box))
           (single-arg (box-arg1 box)))
          ((arg-editable (box-arg2 box))
           (single-arg (box-arg2 box))))))

(defun build-arg-value (arg)
  (cond ((arg-editable arg)
         'editable)
        ((or (numberp (arg-value arg))
             (symbolp (arg-value arg)))
         (arg-value arg))
        ((box-id (arg-value arg)))
        (t (error "aaaah!"))))

;; Hey Andrea, yer prolly interested in this function.
;;
(defun build-box-feature (box view)
  (let ((args (list :x (+ (abs-box-x box view) (round (/ +box-width+ 2)))
                    :y (+ (abs-box-y box view) (round (/ +box-height+ 2)))
                    :color (if (box-selected box) 'cl-user::red 'cl-user::black)
                    :width +box-width+
                    :height +box-height+)))
    (append     
     ;; the hint arrow
     (when (box-pointed-to box)
       (list (make-instance 'redarrow-feature
                            :color 'cl-user::red
                            :value (box-id box)
                            ;; these values come from BOX-LEFT-SIDE in whatsnext.lisp
                            :x (- (abs-box-x box view) 25 +pad+) :y (- (abs-box-y box view) +pad+)
                            :width 20 :height 20)))
     (typecase box
       (x-box
        (list (apply 'make-instance 'x-box-feature
                     :value 'cl-user::box
                     args)))
       (arg-box
        ;; create the text components
        (append
         ;; the black arrows
         (when (typep (arg-value (box-arg1 box)) 'box)
           (list (make-instance 'arrow-feature
                                :x 0 :y 0 :width 0 :height 0
                                :value 'cl-user::left
                                ;; from and to
                                :userprop1 (box-id (arg-value (box-arg1 box)))
                                :userprop2 (box-id box))))
         (when (typep (arg-value (box-arg2 box)) 'box)
           (list (make-instance 'arrow-feature
                                :x 0 :y 0 :width 0 :height 0
                                :value 'cl-user::right
                                ;; from and to
                                :userprop1 (box-id (arg-value (box-arg2 box)))
                                :userprop2 (box-id box))))
         (cons (apply 'make-instance 'arg-box-feature
                      :value (box-id box)
                      ;;left-arg
                      :userprop1 (build-arg-value (box-arg1 box))
                      ;; op
                      :userprop2 (build-arg-value (box-op box))
                      ;; right-arg
                      :userprop3 (build-arg-value (box-arg2 box))
                      args)
               (build-box-contents box view))))
       (answer-box 
        (setf *ab* box)
        (let ((type (if (typep box 'answer-box) 'answer-box-feature 'number-box-feature))
              (accessor  (if (typep box 'answer-box) 'answer 'number)))
          (append
           (list (apply 'make-instance type
                        :value (if (arg-editable (answer box))
                                 (arg-id (answer box))
                                 'cl-user::box)
                        ;; answer
                        :userprop1 (build-arg-value (funcall accessor box)) 
                        args))
           (when (typep (arg-value (funcall accessor box)) 'box)
             (list (make-instance 'arrow-feature
                                  :x 0 :y 0 :width 0 :height 0
                                  :value 'cl-user::center
                                  ;; apparently we can't add our own
                                  ;; slots so we need to use the
                                  ;; userpropX slots.
                                  ;; from and to
                                  :userprop1 (box-id (arg-value (funcall accessor box)))
                                  :userprop2 (box-id box)))))))
       (number-box
        (setf *nb* box)
        (let ((type (if (typep box 'answer-box) 'answer-box-feature 'number-box-feature))
              (accessor  (if (typep box 'answer-box) 'answer 'number)))
          (append
           (list (apply 'make-instance type
                        :value (box-id box)
                        ;; answer
                        :userprop1 (build-arg-value (funcall accessor box)) 
                        args))
           (when (typep (arg-value (funcall accessor box)) 'box)
             (list (make-instance 'arrow-feature
                                  :x 0 :y 0 :width 0 :height 0
                                  :value 'cl-user::center
                                  ;; apparently we can't add our own
                                  ;; slots so we need to use the
                                  ;; userpropX slots.
                                  ;; from and to
                                  :userprop1 (box-id (arg-value (funcall accessor box)))
                                  :userprop2 (box-id box)))))))))))

(defun features-width (features)
  (let ((min (loop for i in features
                   minimize (cl-user::screen-x i)))
        (max (loop for i in features
                   maximize (+ (cl-user::screen-x i)
                               (cl-user::width i)))))
    (when features
      (- max min))))

(defun offset-features (x y features)
  (dolist (i (if (listp features) 
                 features
               (list features)))
    (incf (cl-user::screen-x i) x)
    (incf (cl-user::screen-y i) y))
  features)

(defun make-text-feat (txt &optional (color 'cl-user::black))
  (make-instance 'text-feature
                 :x 0 :y 0 :width (string-width txt +eqn-font+) :height 20
                 :value txt :color color))

(defun build-equations (view)
  "Return a list of features for the equation views"
  (loop for i in (coerce (view-subviews view) 'list)
        append (let ((a (build-equation-features (eqn-view-root i) nil)))
                 (offset-features (abs-x i) (abs-y i) a)
                 a)))

(defun adjust-for-parens (expr children)
  (incf (cl-user::width expr) (string-width "()" +eqn-font+))
  (offset-features (string-width "(" +eqn-font+) 0 children))

(defun inputed (box)
  (with-slots (arg1 op arg2) box
    (cond 
     ((and (arg-editable arg1)
           (arg-editable op)
           (arg-editable arg2))
      (when (and (arg-user-answer arg1)
                 (arg-user-answer op)
                 (arg-user-answer arg2))
        (format nil "~@{~a~}" (arg-user-answer arg1) (arg-user-answer op) (arg-user-answer arg2))))
     ((and (arg-editable arg1)
           (arg-editable op))
      (when (and (arg-user-answer arg1)
                 (arg-user-answer op))
        (format nil "~@{~a~}" (arg-user-answer arg1) (arg-user-answer op))))
     ((and (arg-editable op)
           (arg-editable arg2))
      (when (and (arg-user-answer op)
                 (arg-user-answer arg2))
        (format nil "~@{~a~}" (arg-user-answer op) (arg-user-answer arg2)))))))

;; This isn't pretty.
(defun build-equation-features (tree parens)
  "Tutor2 features."
  (let ((child-parens (not (typep tree 'equal-box))))
    (labels ((stop-p (arg)
                     (and (not (arg-editable arg))
                          (or (symbolp (arg-value arg)) (numberp (arg-value arg))))))
      (etypecase tree
        (arg-box
         (cond 
          ((and (arg-editable (box-arg1 tree))
                (arg-editable (box-op tree))
                (arg-editable (box-arg2 tree)))
           ;; a green box
           (format t "here-1a ~a~%" (box-user-input tree))
           (let* ((r (if (inputed tree)
                         (make-text-feat (inputed tree) 'green)
                       (make-instance 'rectangle-feature
                                      :color 'cl-user::green
                                      :x 0 :y 0 :width 60 :height 20)))
                  (e (make-instance 'expression-feature
                                    :x 0 :y 0 :width (cl-user::width r) :height 20)))
             (when parens (adjust-for-parens e (list r)))
             (list r e)))
          ((and (arg-editable (box-arg1 tree))
                (arg-editable (box-op tree)))
           ;; a green box and some text
           (format t "here-1b ~a~%" tree)
           (let* ((r (if (inputed tree)
                         (make-text-feat (inputed tree) 'green)
                       (make-instance 'rectangle-feature
                                      :color 'cl-user::green
                                      :x 0 :y 0 :width 40 :height 20)))
                  (txt (offset-features (cl-user::width r) 0
                                        (if (stop-p (box-arg2 tree))
                                            (make-text-feat (prin1-to-string (arg-value (box-arg2 tree))))
                                          (build-equation-features (arg-value (box-arg2 tree)) child-parens))))
                  (e  (make-instance 'expression-feature
                                     :x 0 :y 0 :width (+ (cl-user::width r) (features-width txt)) :height 20)))
             (when parens (adjust-for-parens e (append1 txt r)))
             (append1 txt r e)))
          ((and (arg-editable (box-op tree))
                (arg-editable (box-arg2 tree)))
           ;; some text and a green box
           (format t "here-1c ~a~%" tree)
           (let* ((txt (if (stop-p (box-arg1 tree))
                           (make-text-feat (prin1-to-string (arg-value (box-arg1 tree))))
                         (build-equation-features (arg-value (box-arg1 tree)) child-parens)))
                  (r (if (inputed tree)
                         (offset-features (cl-user::width txt) 0 (make-text-feat (inputed tree) 'green))
                       (make-instance 'rectangle-feature
                                      :color 'cl-user::green
                                      :x (cl-user::width txt) :y 0 :width 40 :height 20)))
                  (e (make-instance 'expression-feature
                                    :x 0 :y 0 :width (features-width (append1 r txt)) :height 20)))
             (when parens (adjust-for-parens e (append1 r txt)))
             (append1 txt r e)))
          ((arg-editable (box-arg1 tree))
           ;; some text and a green box
           (format t "here-1d ~a~%" tree)
           (let* ((txt (if (stop-p (box-arg2 tree))
                           (let ((s (format nil "~@{~a~}" (arg-value (box-op tree)) (arg-value (box-arg2 tree)))))
                             (list (make-instance 'text-feature
                                                  :x 0 :y 0 :width (string-width s +eqn-font+) :height 20
                                                  :value s)))
                         (let ((a (build-equation-features (arg-value (box-arg2 tree)) child-parens))
                               (s (prin1-to-string (arg-value (box-op tree)))))
                           (list a (make-instance 'text :x 20 :y 0 :width (string-width s +eqn-font+) :height 20 :value s)))))
                  (r (if (arg-user-answer (box-arg1 tree))
                         (make-text-feat (arg-user-answer (box-arg1 tree)) 'green)
                       (make-instance 'rectangle-feature
                                      :color 'cl-user::green
                                      :x 0 :y 0 :width 40 :height 20)))
                  (e (make-instance 'expression-feature
                                    :x 0 :y 0 :width (+ (cl-user::width r) (cl-user::width txt)) :height 20)))
             (when parens (adjust-for-parens e (append1 txt r)))
             (append1 txt r e)))
          ((arg-editable (box-op tree))
           ;; some text and a green box
           (format t "here-1e ~a~%" tree)
           (let* ((a1 (if (stop-p (box-arg1 tree))
                          (list (make-text-feat (prin1-to-string (arg-value (box-arg1 tree)))))
                        (build-equation-features (arg-value (box-arg1 tree)) child-parens)))
                  (r (offset-features (features-width a1) 0 
                                      (if (arg-user-answer (box-op tree))
                                          (make-text-feat (prin1-to-string (arg-user-answer (box-op tree))) 'green)
                                        (make-instance 'rectangle-feature
                                                       :color 'cl-user::green
                                                       :x 0 :y 0 :width 20 :height 20))))
                  (a2 (offset-features (features-width (append1 a1 r)) 0
                                       (if (stop-p (box-arg2 tree))
                                           (list (make-text-feat (prin1-to-string (arg-value (box-arg2 tree)))))
                                         (build-equation-features (arg-value (box-arg2 tree)) child-parens))))
                  (e (make-instance 'expression-feature
                                    :x 0 :y 0 :width (features-width (append1 a1 a2 r)) :height 20)))
             (when parens (adjust-for-parens e (append1 a1 a2 r)))
             (append1 a1 a2 r e)))
          ((arg-editable (box-arg2 tree))
           ;; some text and a green box
           (format t "here-1f ~a~%" tree)
           (let* ((txt (if (stop-p (box-arg1 tree))
                           (list (make-text-feat (format nil "~@{~a~}" (arg-value (box-arg1 tree)) (arg-value (box-op tree)))))
                         (let ((a (build-equation-features (arg-value (box-arg1 tree)) child-parens)))
                           (append1 a (make-text-feat (prin1-to-string (arg-value (box-op tree))))))))
                  (r (if (arg-user-answer (box-arg2 tree))
                         (offset-features (cl-user::width txt) 0 (make-text-feat (arg-user-answer (box-arg2 tree)) 'green))
                       (make-instance 'rectangle-feature
                                      :color 'cl-user::green
                                      :x (cl-user::width txt) :y 0 :width 40 :height 20)))
                  (e (make-instance 'expression-feature
                                    :x 0 :y 0 :width (+ (cl-user::width r) (features-width txt)) :height 20 :userprop1 (box-pointed-to tree))))
             (when parens (adjust-for-parens e (append1 txt r e)))
             (append1 txt r e)))
          ((and (stop-p (box-arg1 tree))
                (stop-p (box-arg2 tree)))
           (format t "here1 ~a~%" tree)
           (let* ((txt (make-text-feat (format nil "~@{~a~}" (box-left tree) (arg-value (box-op tree)) (box-right tree))))
                  (e (make-instance 'expression-feature
                                    :x 0 :y 0 :width (cl-user::width txt) :height 20 :userprop1 (box-pointed-to tree))))
             (when parens (adjust-for-parens e (list txt)))
             (append1 txt e)))
          ((and (stop-p (box-arg1 tree))
                (not (stop-p (box-arg2 tree))))
           (format t "here2 ~a~%" tree)
           (let* ((arg1 (make-text-feat (format nil "~@{~a~}" (box-left tree) (arg-value (box-op tree)))))
                  (arg2 (build-equation-features (box-right tree) child-parens))
                  (e (make-instance 'expression-feature
                                    :x 0 :y 0 :width (+ (features-width (list arg1)) (features-width arg2)) :height 20 :userprop1 (box-pointed-to tree))))
             (offset-features (cl-user::width arg1) 0 arg2)
             (when parens (adjust-for-parens e (append1 arg1 arg2)))
             (append1 arg1 arg2 e)))
          ((and (not (stop-p (box-arg1 tree)))
                (stop-p (box-arg2 tree)))
           (format t "here3 ~a~%" tree)
           (let* ((txt (make-text-feat (format nil "~@{~a~}" (arg-value (box-op tree)) (box-right tree))))
                  (left (build-equation-features (box-left tree) child-parens))
                  (e (make-instance 'expression-feature
                                    :x 0 :y 0 :width (+ (features-width (list txt)) (features-width left)) :height 20 :userprop1 (box-pointed-to tree))))
             (offset-features (features-width left) 0 (list txt))
             (when parens (adjust-for-parens e (append1 txt left)))
             (append1 txt e left)))
          (t
           (format t "here4 ~a~%" tree)
           (let* ((arg1 (build-equation-features (box-left tree) child-parens))
                  (arg2 (build-equation-features (box-right tree) child-parens))
                  (op (make-text-feat (prin1-to-string (arg-value (box-op tree)))))
                  (e  (make-instance 'expression-feature
                                     :x 0 :y 0 :width (+ (features-width arg1) (features-width arg2) (cl-user::width op)) :height 20 :userprop1 (box-pointed-to tree))))
             (offset-features (features-width arg1) 0 (list op))
             (offset-features (+ (features-width arg1) (cl-user::width op)) 0 arg2)
             (when parens (adjust-for-parens e (append1 arg1 arg2 op)))
             (append1 arg1 arg2 op e)))))
        (answer-box
         (build-equation-features (arg-value (answer tree)) child-parens))
        (x-box
         (list (make-text-feat "x")))))))
     
;; (defun build-equation-features (tree)
;;   (let ((parts (compile-eqn tree nil nil nil nil)))
;;     (


(defun build-tree-features (tree view)
  (labels ((doit (root)
                 (apply 'nconc (build-box-feature root view)
                        (mapcar #'doit (only-boxes (children root))))))
    (when tree
      (doit tree))))

(defun build-current-state-features ()
  (build-tree-features (tutor-eqn-root *tutor*) (tutor-eqn-view *tutor*)))

(defun build-previous-state-features ()
  (build-tree-features (last-eqn) (tutor-last-eqn-view *tutor*)))

(defun build-message-features ()
  (let ((txt (cond ((null *message-widget*)
                    nil)
                   ((typep *message-widget* 'static-text-dialog-item)
                    (list *message-widget*))
                   (t 
                    (coerce (view-subviews *message-widget*) 'list)))))
    ;; TODO: color
    (mapcar (lambda (v)
              (make-instance 'text-feature
                             :value (string-downcase (dialog-item-text v))
                             :x (abs-x v)
                             :y (abs-y v)
                             :width (view-width v)
                             :height (view-height v)))
            txt)))

(defun build-status-features ()
  ;; TODO
  ;; money, problem #
  )

(defun build-input-box-features ()
  ;; This is maybe kinda cheating. Since the input dialog is a
  ;; seperate window, find its position relative to *window* and just
  ;; say that's where it is. This screws things like focus up, i suspect.
  (when *input-dialog*
    (let ((x (- (view-x *input-dialog*) (view-x *window*)))
          (y (- (view-y *input-dialog*) (view-y *window*)))
          (items (coerce (view-subviews *input-dialog*) 'list)))
      ;; for now just create the dialog. We need to add the text input
      ;; box, the okay button, the text above it and the red close
      ;; button.
      (cons
       (make-instance 'input-dialog-feature
                      :value 'cl-user::input-dialog-feature ; fixme
                      :x x
                      :y y
                      :width (view-width *input-dialog*)
                      :height (view-height *input-dialog*))
       (mapcar (lambda (b)
                 (make-instance (etypecase b
                                  (button-dialog-item 'button-feature)
                                  (static-text-dialog-item 'text-feature)
                                  (editable-text-dialog-item 'text-feature))
                                :value (string-downcase (dialog-item-text b))
                                :x (+ x (view-x b)) :y (+ y (view-y b))
                                :width (view-width b) :height (view-height b)))
               items)))))

(defun build-calc-features ()
  (when (and calc::*calc-window*
             (window-shown-p calc::*calc-window*))
    (let* ((w calc::*calc-window*)
           (x (- (view-x w) (view-x *window*)))
           (y (- (view-y w) (view-y *window*)))
           (items (coerce (all-subviews w 'button-dialog-item 'static-text-dialog-item) 'list)))
      (cons 
       (make-instance 'calculator-feature
                      :x x
                      :y y
                      :width (view-width w)
                      :height (view-height w))
       (mapcar (lambda (b)
                 (make-instance (if (typep b 'button-dialog-item) 'button-feature  'text-feature)
                                :value (string-downcase (dialog-item-text b))
                                :x (+ x (view-x b)) :y (+ y (view-y b))
                                :width (view-width b) :height (view-height b)))
       items)))))
    
(defmethod cl-user::build-features-for ((self tutor-window) vismod ;; (vis-mod tutor-vision-module)
                                        )
  "The entry point."
  (nconc 
   (build-instruction-features)
   (build-button-features)
   (if (tutor1-p)
       (nconc
         (build-current-state-features)
         (build-previous-state-features))
     (build-equations (tutor-eqn-view *tutor*)))
   (build-message-features)
   (build-status-features)
   (build-input-box-features)
   (build-calc-features)))

(defun draw-all-features (features)
  "Create a window and draw rectangles for all the features"
  (let ((w (make-instance 'window
                          :view-size (make-point (view-width *window*)
                                                 (view-height *window*))
                          :window-type :document)))
    (dolist (f features)
      (with-slots (cl-user::screen-x cl-user::screen-y cl-user::width cl-user::height cl-user::val) f
        (unless (typep f 'expression-feature)
          (draw-rect w 
                     cl-user::screen-x cl-user::screen-y (+ cl-user::screen-x cl-user::width) (+ cl-user::screen-y cl-user::height))
        (when (and (typep f 'text-feature)
                   (stringp (cl-user::val f)))
          (draw-text-with-font w cl-user::screen-x cl-user::screen-y cl-user::val +eqn-font+))
        )))))


(defun append1 (&rest lists)
  "append the lists together. If elements are not
  in a list, then wrap it in a list and append it."
  (apply 'append (mapcar (lambda (l)
                          (if (and (atom l)
                                   (not (null l)))
                              (list l)
                            l))
                        lists)))

(defclass tutor2-eqn-view (view)
  ((root :initform nil :accessor eqn-view-root))
  (:documentation "A special view that points to the equation that it contains. Used by the actr-pm stuff."))

(defun abs-box-x (box view)
  (+ (abs-x view)
     (box-x box)))
;;         (point-h (view-scroll-position view)))))

(defun abs-box-y (box view)
  (+ (abs-y view)
     (box-y box)))
;;         (point-v (view-scroll-position view)))))


;; Ver bad hack to get the ID of an editable box.
;; Shawn, lte me know when you produce better dump-screens! 

(defun get-proper-id (box)
  (cond ((arg-editable (box-arg1 box))
         (arg-id (box-arg1 box)))
        ((arg-editable (box-arg2 box))
         (arg-id (box-arg2 box)))
        ((arg-editable (box-op box))
         (arg-id (box-op box)))
        (t
         (box-id box))))



(defmethod view-key-event-handler ((view tutor::tutor-window) 
                                   (char character))
  (if (not cl-user::*actr-enabled-p*)
    (call-next-method)
    (progn
      (cond ((graphic-char-p char) 
             (execute-command :type char))
            ((char= char #\-) 
             (execute-command :type char))
            ((char= char #\return)
             (submit))
            ((char= char #\backspace)
             (ed-rubout-char view))
            )
      (view-draw-contents view))))

;(defmethod view-key-event-handler ((view ccl::get-string-dialog) 
;                                   (char character))
;  (if (not cl-user::*actr-enabled-p*)
;    (call-next-method)
;    (progn
;      (cond ((graphic-char-p char) 
;             (execute-command :type char))
;            ((char= char #\-) 
;             (execute-command :type char))
;            ((char= char #\return)
;             (submit))
;            ((char= char #\backspace)
;             (ed-rubout-char view))
;            )
;     (view-draw-contents view))))

(defun submit ()
  "Submits the input dialog"
  (when *input-dialog*
    (execute-command :submit nil)))