;;; What's next functionality

;; find an operation that can be performed and tell the user about it.

(in-package :tutor)

;; Accelerator keys

(defvar *whatsnext-arrows* nil)

(defvar *whatsnext-hint* nil
  "")

(defun init-whatsnext-arrows ()
  (setf *whatsnext-arrows* nil))

(defun box-left-side (b)
  (values (- (+ (point-h (view-position (tutor-eqn-view *tutor*))) (box-x b))
	     (point-h (view-scroll-position (tutor-eqn-view *tutor*)))
	     ;; Make a bit of a space between the arrow tip and the box
	     25
	     +pad+)
	  (+ (- (+ (point-v (view-position (tutor-eqn-view *tutor*))) (box-y b))
		(point-v (view-scroll-position (tutor-eqn-view *tutor*))))
	     +pad+)))

(defun box-clear-arrows (root)
  (setf (box-pointed-to root) nil)
  (mapc #'box-clear-arrows (only-boxes (children root))))

(defun clear-arrows ()
  ;; clear the box's arrow cache
  (box-clear-arrows (tutor-eqn-root *tutor*))
  (when *whatsnext-arrows*
    (apply #'remove-subviews (tutor-eqn-area-view *tutor*) *whatsnext-arrows*)
    (setf *whatsnext-arrows* nil)))

(defun point-to-boxes (boxes)
;;   (clear-arrows)
;;   (setf *whatsnext-arrows*
	(loop for b in boxes
	      do (setf (box-pointed-to b) t))
        (refresh-eqn))
;; 	      collect (multiple-value-bind (x y) (box-left-side b)
;; 			(draw-arrow (tutor-eqn-area-view *tutor*) x y 20 20 *red-color*)))))

(defun valid-operation-patterns (patterns root)
  (labels ((valid-operation (pattern root)
             (multiple-value-bind (matched alist) (matcher pattern root)
               ;; (format t "alist: ~a rest: ~a~%" alist rest)
               (when matched
                 (values (position pattern patterns :test #'equal) alist)))))
    (dolist (p patterns)
      (multiple-value-bind (i v) (valid-operation p root)
        (if (and i v)
            (return-from valid-operation-patterns (values i v)))))))

(defun minus-case-is-allowed (i)
  "Return T if this problem set allows minus hints of the Ith kind."
  (find (case i (0 :number) (1 :x) (2 :subexpr)) (hint-types (tutor-problem-set *tutor*))))

(defun check-operation (opn root)
  "Return a list of boxes the user must select, if this succeeds
on ROOT."
  (let* ((data (find-operation opn))
         (pats (operation-patterns data)))
    (multiple-value-bind (i v) (valid-operation-patterns pats root)
      (when (and i v
                 (if (eq opn 'minus)
                     (minus-case-is-allowed i)
                   t))
        ;;(format t "~a ~a~%" i v)
        (plist-vals v (nth i (operation-selects data)))))))

(defun walk-boxes (opn root)
  (when (typep root 'box)
    (or (check-operation opn root)
        (loop for i in (children root)
              do (iflet boxes (walk-boxes opn i)
                   (return boxes))))))

(defun find-operation-on-boxes (operations)
  (dolist (i operations)
    (iflet w (walk-boxes i (tutor-eqn-root *tutor*))
      (return-from find-operation-on-boxes
        (values i w)))))

(defun can-do-more (&optional not-minus)
  "Return T if something more can be done to the problem. NIL
  otherwise. When NOT-MINUS is T, don't include the MINUS
  operation when checking for the next operation."
  (when (find-operation-on-boxes (if not-minus
                                   (remove 'minus (all-operations nil))
                                   (all-operations nil)))
    t))

(defun click-gb ()
  (tutor-green-box-hints *tutor*))
;;   (let* ((a (editables-answers (tutor-eqn-root *tutor*)))
;;          (l (length a)))
;;     (format nil "Click on the green ~a and input the value~p ~a." 
;;             (if (> l 1) "boxes" "box")
;;             l
;;             (if (> l 1)
;;                 (format nil "~{~#[~;or ~] ~a~^, ~}" a)
;;               (first a)))))

(defun box-text (singular &optional number-box)
  (if (tutor1-p)
      (if singular
          "box"
        "boxes")
    (if number-box
        "variable"
      (if singular
          "operator"
        "operators"))))

(defun whats-next (root &optional (operations (all-operations nil)))
  (cond
   ;; if we are in fill-in-the-boxes state, then tell them to click on a green box
   ;; if all the green boxes are filled in, then tell them to click the submit box
   ((tutor-submit-enabled *tutor*)
     (if (input-filled-in-p root)
	 (unless (correct-answer-p)
           (values (click-gb)
                   "Your answer is incorrect."))
       (click-gb)))
   ((correct-problem-answer-p)
    (format nil "Your Answer is correct.~%Click the Next Prob. button~a." 
            (if *simplified-ui*
                ""
              " (or press the 'n' key)")))
   ;; if not in fill-in-the-boxes state:
   ;; if there are selected boxes, see if an operation can be done on
   ;; them. if it can, tell them to perform it. If not tell them to
   ;; unselect the boxes as no operation can be performed.
   (t
    (multiple-value-bind (opn boxes) (find-operation-on-boxes operations)
      (if opn
	  (progn
	    (point-to-boxes boxes)
	    (cond ((null (set-exclusive-or boxes (selected-boxes)))
		   (format nil "Click the ~a button~a." (op-name (find-operation opn))
                           (if *simplified-ui*
                               ""
                             (format nil "~%(or press the '~C' key)" (operation-quick-key (find-operation opn))))))
		  ;;(< (length boxes) (length (selected-boxes)))
		  ((set-difference (selected-boxes) boxes)
		   (format nil "Unselect the ~a not pointed to by the arrow~p." 
                           (box-text (= 1 (length (set-difference (selected-boxes) boxes))))
                           (length (set-difference (selected-boxes) boxes))))
		  ((set-difference boxes (selected-boxes))
		   (format nil "Click the ~a pointed to by the arrow~p."
                           (box-text (= 1 (length boxes)) 
                                     (typep (first boxes) 'number-box)) 
                           (length boxes)))))
;; 	(if (selected-boxes)
;; 	    (format nil "No operation can be done.~%Use the <- and -> buttons (or the left and right arrow keys)~%to review your work and correct the mistake.")
;;           (if (correct-problem-answer-p)
;;               (format nil "Your Answer is correct.~%Click the Next Problem button (or press the 'n' key).")
            (format nil "Your answer is incorrect.~%Use the <- and -> buttons (or the left and right arrow keys)~%to review your work and correct the mistake."))))))

(defun do-whats-next (hint-type)
  (clear-arrows)
  (let* ((ret (multiple-value-list (whats-next (tutor-eqn-root *tutor*)
                                               (ecase hint-type
                                                 (:all
                                                   (all-operations nil))
                                                 (:base (remove 'minus (all-operations nil)))))))
         (txt (first ret))
         (header (second ret))
         (click (click-instruction-text))
         (hint (if click (format nil "~a~%~a" txt click) txt)))
    (unless (auto-whats-next-mode)
      ;; incentives
      (decf (tutor-problem-score *tutor*) +hint-points+)
      (history-add-whats-next txt))
    ;; Only overwrite the error text if the hint has changed. Yes, its
    ;; a hack.
    (when (or header 
              (not (string-equal hint *hint-text*)))
      (set-message-text header))
    (set-hint-text hint)))
